/**
 * To make up for a lack in haxe, this is a standard assertion
 * class.
 * It's behaviour is to trace that an assertion occured and then
 * to through an exception -- hopefully something can display
 * the exception (like Flash Debug player)
 */
class Assert
{
	public function new()
	{
	}
	
	static public function fail( msg : String, ?pos:haxe.PosInfos )
	{
		trace( "Assertion in " + pos.className + "::" +pos.methodName + ": " + msg, pos );
		throw new Assert();	//something that nobody is likely catching...
	}
	
	static public function isTrue( cond : Bool, ?pos:haxe.PosInfos )
	{
		if( !cond )
			fail( "isTrue",pos );
	}
	
	static public function unreachable( ?pos:haxe.PosInfos)
	{
		fail( "unreachable", pos);
	}
	
	static public function is( obj : Dynamic, type : Dynamic, ?pos:haxe.PosInfos )
	{
		if( !Std.is( obj, type ) )
			fail( "is(" + obj + "," + type + ")", pos );
	}
	
	static public function notNull( obj : Dynamic, ?pos:haxe.PosInfos )
	{
		if( obj == null )
			fail( "notNull", pos );
	}
	
	static public function isNull( obj : Dynamic, ?pos:haxe.PosInfos )
	{
		if( obj != null )
			fail( "isNull", pos );
	}
	
	/**
	 * Used to indicate the funciton is never meant to be called, it is rather
	 * a placeholder for a function to be defined in a derived class.
	 */
	static public function pureVirtual( ?pos:haxe.PosInfos )
	{
		fail( "pure virtual called", pos );
	}
}