
import flash.net.SharedObject;

/**
 * A class to store persistant data with the program.
 *
 * Any SharedObject serializable data type is supported, to what extent that supports
 * custom classes is not known.  The Flash 9 SharedObject is not so clear
 * on exactly what is supported.
 */
class Persistent
{
	static var so : SharedObject;
		
	static function init()
	{
		so = SharedObject.getLocal( "data" );
	}
	
	/**
	 * Obtains a persistent data. 
	 *
	 * NOTE: The stored "null" value will also result in this returning the default value.
	 *
	 * @param	name	[in] the name of the data to retrieve
	 * @param	def	[in] the value to return if none was specified previously
	 */
	static public function get<T>( name : String, def : T ) : T
	{
		if( so == null ) 
			init();
			
		if( untyped so.data[name] == null )
			return def;
		return untyped so.data[name];
	}
	
	/**
	 * Sets a persistent data.
	 *
	 * NOTE: As null an not-set are treated the same, calling this with a null value
	 * is equivalent to removing the value.
	 *
	 * @param	name	[in] name of the value to set
	 * @param	val	[in] the content of the value to set
	 */
	static public function set<T>( name : String, val : T )
	{
		if( so == null )
			init();
			
		untyped so.data[name] = val;
		so.flush();	//always write
	}
}
