/**
 * Creates a range iterator given a starting point, a target, and a stepping
 * direction/amount.
 *
 * Done via an M4 macro since haxe doesn't handle templated argumetns very well...
 */
 
	//end of macro




class RangeIterI
{
	var at : Int  ;	//where were at now
	var dir : Int  ;	//which direction/step we're taking
	var target : Int  ;	//to where we're heading (may not be included in results if no within step)
	
	/**
	 * Creates a new inclusive iterator.
	 *
	 * @param	from	[in] where to start from
	 * @param	to	[in] up until and included this value
	 * @param	step	[in] amount to step by (positive or negative) (default = +1, or -1, based on from/to)
	 * @param	allowEmpty [in] causes an assertion if the range is empty (default = true)
	 */
	public function new( from : Int  , to : Int  , ?step : Null<Int  >, ?allowEmpty : Null<Bool> )
	{
		at = from;
		target = to;
		
		if( step == null )
			step = if( at < target ) 1 else -1;
		dir = step;
		
		//check for infinite cases
		Assert.isTrue( 
			(at < target && dir > 0) 
			|| (at > target && dir < 0)
			|| (at == target && dir != 0) 	//special case for a single entry range
			);
			
		//check for empty cases
		if( allowEmpty != null && !allowEmpty )
			Assert.isTrue( hasNext() );
	}
	
	public function hasNext() : Bool
	{
		if( dir < 0 )
			return at >= target;
		return at <= target;
	}
	
	public function next() : Int  
	{
		var ret = at;
		at += dir;
		return ret;
	}
}

