package mathx;

//real version doesn't work with AS3, and it is always too slow...
typedef FlashMatrix = Matrix<Int>;

#if never-flash

package mathx;
import flash.display.BitmapData;

/**
 * A matrix class specialized for Flash
 *
 * This works with Int's of up to 24bits in size, unsigned only!!!
 */
class FlashMatrix extends Matrix<Int>
{
	private var fdata : BitmapData;
	
	public function get( x : Int, y : Int ) : Int
	{
		//assert validIndex(x,y);
		return fdata.getPixel( x, y );
	}
	
	public function set( x : Int, y : Int, val : Int )
	{
		//assert validIndex(x, y );
		fdata.setPixel( x, y, val );
	}
	
	//only static functions can create a new matrix
	private function new()
	{
		super();
	}
	
	static public function fromArrayMat( mat : Array<Array<Int>> ) : FlashMatrix
	{	
		//assert mat.length > 0
		//assert mat[0].length > 0
		
		var m = new FlashMatrix();
		m.sizeX = mat[0].length;
		m.sizeY = mat.length;
		m.fdata = new BitmapData( m.sizeX, m.sizeY, false );
		
		for( i in 0...m.sizeY )
		{
			//assert mat[i] == sizeX
			for( j in 0...m.sizeX )
				m.fdata.setPixel( j, i, mat[i][j] );
		}
		
		return m;
	}
	
	/**
	 * Creates a matrix with the default value given
	 */
	static public function create( cols : Int, rows : Int, init : Int ) : FlashMatrix
	{
		var m = new FlashMatrix();
		m.sizeX = cols;
		m.sizeY = rows;
		m.fdata = new BitmapData( m.sizeX, m.sizeY, false, init );
		return m;
	}
	
	/**
	 * Does a copy of the matrix.  The internal items are not copied, as this
	 * is meant for fundamentals (where a copy is always taken
	 */
	public function copy() : FlashMatrix
	{
		var m = new FlashMatrix();
		m.sizeX = sizeX;
		m.sizeY = sizeY;
		m.fdata = fdata.clone();
		return m;
	}


	/**
	 * Extracts the row from the matrix
	 */
	public function extractRow( y : Int ) : Array<Int>
	{
		//assert bounds
		var r = new Array<Int>();
		for( i in 0...sizeX )
			r.push( fdata.getPixel( i, y ) );
		return r;
	}	 
	
	/**
	 * Flips the matrix y[n] <=> y[0].  Returns a new matrix, original is not modified
	 */
	public function createFlip() : Matrix<Int>
	{
		//TODO: Optimize for Flash!
		
		var m = new FlashMatrix();
		m.sizeX = sizeX;
		m.sizeY = sizeY;
		m.fdata = new BitmapData( m.sizeX, m.sizeY, false );
		
		for( y in 0...m.sizeY )
			for( x in 0...m.sizeX )
				m.fdata.setPixel( x,y, fdata.getPixel32( x, m.sizeY -y-1 ) );
		return m;
	}
}

#end
