package mathx;

/**
 * Geometry-like functions, mainly ported from ssim/math.h
 */
class Geometry
{
	static public function distanceSqr( a : MatPointF, b : MatPointF ) : Float
	{
		return (b.x-a.x)*(b.x-a.x) + (b.y-a.y)*(b.y-a.y);
	}

	static public function norm( a : MatPointF ) : Float
	{
		return Math.sqrt( a.x*a.x + a.y*a.y );
	}

	static public function distance( a : MatPointF, b : MatPointF ) : Float
	{
		return Math.sqrt( distanceSqr( a, b ) );
	}

	static public function unit( a :MatPointF ) : MatPointF
	{
		var len = Math.sqrt( a.x * a.x + a.y * a.y );
		return MatPointF.at( a.x/len, a.y/len );
	}
	
	/** returns the angle between this line and the positive xAxis
 	* range = [ -M_PI, M_PI ]
 	*/
	static public function xAxisAngle( a : MatPointF, b : MatPointF ) : Float
	{
		var n = b.sub( a );
		var raw = Math.asin( n.y / norm( n ) );
		
		if( n.x > 0 )
			return raw;
		else
		{
			if( n.y > 0 )
				return MathUtil.PI - raw;
			else
				return -(MathUtil.PI + raw);
		}
	}
	
	//TODO: Optimize (this version is when origin is truly the origin)
	static public function xAxisAngleV( a : MatPointF ) : Float
	{
		return xAxisAngle( MatPointF.at( 0, 0 ), a );
	}
	
	static public function vFromAngle( a : Float ) : MatPointF
	{
		return MatPointF.at(  Math.cos( a ), Math.sin( a ) );
	}
}
