package mathx;

class MatrixGeom
{
	//grid based directional items, in clockwise order
	static public var dirs8 : Array<MatPoint> = [
		MatPoint.at( -1, -1 ),
		MatPoint.at( 0, -1 ),
		MatPoint.at( 1, -1 ),
		MatPoint.at( 1, 0 ),
		MatPoint.at( 1, 1 ),
		MatPoint.at( 0, 1 ),
		MatPoint.at( -1, 1 ),
		MatPoint.at( -1, 0 ),
		];
		
	static public var dirs4 : Array<MatPoint> = [
		MatPoint.at( 0, -1 ),
		MatPoint.at( 1, 0 ),
		MatPoint.at( 0, 1 ),
		MatPoint.at( -1, 0 ),
		];
		
	/**
	 * Given the current direction and location determine the next
	 * grid coordinate which will be encountered (excluding this one)
	 *
	 * This includes a diagonal tolerance, since excepting for the rare
	 * perfect condition, a diagonal would never be met first, only lateral
	 * neighbours.  Though this needs to be tempered based on angle,
	 * so that true lateral motion remains so.  (This can simply be the direction
	 * itself in the formula below)
	 *
	 * @param	at	[in] starting location
	 * @param	dir	[in] unit vector direction to move
	 * @return [out] where you end up
	 */		
	static public function nextGridFrom( at : MatPointF, dir : MatPointF ) : MatPoint
	{
		//assume a unit dir vector, assert can be too costly here...
		
		var cur = at.floorDemote();
		//special case if not moving (exact 0 compare okay, since unit vector)
		if( dir.x == 0 && dir.y == 0 )
			return cur;
			
		var target = at.add( dir ).floorDemote();
		if( target.x == cur.x && target.y == cur.y )
		{
			//oops, special condition that a 1 offset is not enough to determine target
			//2 is enough, technically root(2) should be also...
			target = at.add( dir ).add( dir ).floorDemote();
		}
		
		return target;
	}
	
	static public function nextGridBase( dir : MatPointF ) : MatPoint
	{
		//assume a unit dir vector, assert can be too costly here...
		
		//special case if not moving (exact 0 compare okay, since unit vector)
		if( dir.x == 0 && dir.y == 0 )
			return MatPoint.at( 0, 0 );
			
		//unlike From version we will always be in the next grid here
		return dir.add( MatPointF.at( 0.5, 0.5 ) ).floorDemote();
	}
	
}
