package mathx;

/**
 * Simple array utilities that seem to be missing from haxe...?
 */
class MatrixUtil
{
	
	/**
	 * Compares two matrices -- mainly just here for the test units
	 */
	static public function equals<T>( a : Matrix<T>, b : Matrix<T> ) : Bool
	{
		if( a.sizeX != b.sizeX
			|| a.sizeY != b.sizeY )
			return false;
			
		for( mp in a.xOrderIter() )
			if( a.get( mp.x, mp.y ) != b.get( mp.x, mp.y ) )
				return false;
			
		return true;
	}
	
	/**
	 * Rotates the matrix clockwise 90 degrees (transform)
	 */
	static public function rotate90C<T>( wl : Matrix<T> ) : Matrix<T>
	{
		//TODO: merge into Matrix itself so that initial creation will do copy
		var nwl = Matrix.create( wl.sizeY, wl.sizeX, wl.get(0,0) );
		for( x in 0...wl.sizeX )
			for( y in 0...wl.sizeY )
				nwl.set(  wl.sizeY-y-1, x, wl.get( x, y ) );

		return nwl;
	}
	
	/**
	 * Internal seed fill which takes the desired offsets as input
	 *
	 * TODO: Generate specific Fills via M4
	 */
	static private function seedFill<T>( colorMap : Matrix<T>, 
		seeds : FastList<MatPoint>, match : T, fill : T, offsets : Array<Array<Int>>,
		?shadowMap : Matrix<T>, ?shadowFill : T )
	{
		//trying to cache MatPoint objects slows this down slightly (using List structure)
		//however, using individual lists for X/Y does help!
		
		//PERFORMANCE: In AS3 Compiler Array is far faster than List!
		var sX = new FastList<Int>();
		var sY = new FastList<Int>();
		for( i in seeds )
		{
			sX.push( i.x );
			sY.push( i.y );
		}
		
		while( sX.length > 0 )
		{
			var ccX = sX.pop();
			var ccY = sY.pop();
			
			colorMap.set( ccX, ccY, fill );
			for( off in 0...offsets.length )
			{
				var nx = ccX + offsets[off][0];
				var ny = ccY + offsets[off][1];
				
				if( //colorMap.validIndex(nx,ny)	//PERFORMANCE
					nx >= 0 && ny >= 0
					&& nx < colorMap.sizeX	&& ny < colorMap.sizeY
					&& colorMap.get(nx,ny) == match )
				{
					colorMap.set( nx, ny, fill );
					sX.push( nx );
					sY.push( ny );
					
					//virtually no performance hit by this repeated comparison
					if( shadowMap != null )
						shadowMap.set( nx, ny, shadowFill );
				}
			}
			
		}	//while(open)
	}
	
		//end of seed-Fill-TPL
	
	
	
		
	
	
	
	
		
		
		
		
	static private function seedFill_4<T>( colorMap : Matrix<T>, 
		seeds : FastList<MatPoint>, 
		match1 : T,  
		fill : T, 	?shadowMap : Matrix<T>, ?shadowFill : T )
	{
		//PERFORMANCE: In AS3 Compiler Array is far faster than List! (TODO: choose based on compiler options?)
		var sX = new FastList<Int>();
		var sY = new FastList<Int>();
		for( i in seeds )
		{
			sX.push( i.x );
			sY.push( i.y );
			colorMap.set( i.x, i.y, fill );	//pre-fill these (to avoid duplicate fills later) (TODO: promote to others)
		}
		
		while( sX.length > 0 )
		{
			var ccX = sX.pop();
			var ccY = sY.pop();
			
			colorMap.set( ccX, ccY, fill );
			for( off in 0... 4 )
			{
				//PERFORANCE: No static arrays, and minimize if comparisons
				var nx = ccX;
				var ny = ccY;
				
				switch( off )
				{
					case 0:
						nx--;
					case 1:
						nx++;
					case 2:
						ny--;
					case 3:
						ny++;
				}
	 
				
				//Strangely, this is better than putting continues in the switch
				//statement on possibly invalid items (at least for size 8, 4 the continues were fine)
				if( //colorMap.validIndex(nx,ny)	//PERFORMANCE
					nx >= 0 && ny >= 0
					&& nx < colorMap.sizeX	&& ny < colorMap.sizeY
					&& ( 
						colorMap.get(nx,ny) == match1
						 
					) )
				{
					colorMap.set( nx, ny, fill );
					sX.push( nx );
					sY.push( ny );
					
					//virtually no performance hit by this repeated comparison
					if( shadowMap != null )
						shadowMap.set( nx, ny, shadowFill );
				}
			}
		}	//while(open)
	}
	 
	
		
		
		
		
	static private function seedFill_2_4<T>( colorMap : Matrix<T>, 
		seeds : FastList<MatPoint>, 
		match1 : T, match2 : T,  
		fill : T, 	?shadowMap : Matrix<T>, ?shadowFill : T )
	{
		//PERFORMANCE: In AS3 Compiler Array is far faster than List! (TODO: choose based on compiler options?)
		var sX = new FastList<Int>();
		var sY = new FastList<Int>();
		for( i in seeds )
		{
			sX.push( i.x );
			sY.push( i.y );
			colorMap.set( i.x, i.y, fill );	//pre-fill these (to avoid duplicate fills later) (TODO: promote to others)
		}
		
		while( sX.length > 0 )
		{
			var ccX = sX.pop();
			var ccY = sY.pop();
			
			colorMap.set( ccX, ccY, fill );
			for( off in 0... 4 )
			{
				//PERFORANCE: No static arrays, and minimize if comparisons
				var nx = ccX;
				var ny = ccY;
				
				switch( off )
				{
					case 0:
						nx--;
					case 1:
						nx++;
					case 2:
						ny--;
					case 3:
						ny++;
				}
	 
				
				//Strangely, this is better than putting continues in the switch
				//statement on possibly invalid items (at least for size 8, 4 the continues were fine)
				if( //colorMap.validIndex(nx,ny)	//PERFORMANCE
					nx >= 0 && ny >= 0
					&& nx < colorMap.sizeX	&& ny < colorMap.sizeY
					&& ( 
						colorMap.get(nx,ny) == match1
						 || colorMap.get(nx,ny) == match2  
					) )
				{
					colorMap.set( nx, ny, fill );
					sX.push( nx );
					sY.push( ny );
					
					//virtually no performance hit by this repeated comparison
					if( shadowMap != null )
						shadowMap.set( nx, ny, shadowFill );
				}
			}
		}	//while(open)
	}
	 
	
		
		
		
		
	static private function seedFill_8<T>( colorMap : Matrix<T>, 
		seeds : FastList<MatPoint>, 
		match1 : T,  
		fill : T, 	?shadowMap : Matrix<T>, ?shadowFill : T )
	{
		//PERFORMANCE: In AS3 Compiler Array is far faster than List! (TODO: choose based on compiler options?)
		var sX = new FastList<Int>();
		var sY = new FastList<Int>();
		for( i in seeds )
		{
			sX.push( i.x );
			sY.push( i.y );
			colorMap.set( i.x, i.y, fill );	//pre-fill these (to avoid duplicate fills later) (TODO: promote to others)
		}
		
		while( sX.length > 0 )
		{
			var ccX = sX.pop();
			var ccY = sY.pop();
			
			colorMap.set( ccX, ccY, fill );
			for( off in 0... 8 )
			{
				//PERFORANCE: No static arrays, and minimize if comparisons
				var nx = ccX;
				var ny = ccY;
				
				switch( off )
				{
					case 0:
						nx--;
						ny--;
					case 1:
						nx--;
					case 2:
						nx--;
						ny++;
					case 3:
						ny--;
					case 4:
						ny++;
					case 5:
						nx++;
						ny--;
					case 6:
						nx++;
					case 7:
						nx++;
						ny++;
				}
	 
				
				//Strangely, this is better than putting continues in the switch
				//statement on possibly invalid items (at least for size 8, 4 the continues were fine)
				if( //colorMap.validIndex(nx,ny)	//PERFORMANCE
					nx >= 0 && ny >= 0
					&& nx < colorMap.sizeX	&& ny < colorMap.sizeY
					&& ( 
						colorMap.get(nx,ny) == match1
						 
					) )
				{
					colorMap.set( nx, ny, fill );
					sX.push( nx );
					sY.push( ny );
					
					//virtually no performance hit by this repeated comparison
					if( shadowMap != null )
						shadowMap.set( nx, ny, shadowFill );
				}
			}
		}	//while(open)
	}
	 	
	
		
		
		
		
	static private function seedFill_2_8<T>( colorMap : Matrix<T>, 
		seeds : FastList<MatPoint>, 
		match1 : T, match2 : T,  
		fill : T, 	?shadowMap : Matrix<T>, ?shadowFill : T )
	{
		//PERFORMANCE: In AS3 Compiler Array is far faster than List! (TODO: choose based on compiler options?)
		var sX = new FastList<Int>();
		var sY = new FastList<Int>();
		for( i in seeds )
		{
			sX.push( i.x );
			sY.push( i.y );
			colorMap.set( i.x, i.y, fill );	//pre-fill these (to avoid duplicate fills later) (TODO: promote to others)
		}
		
		while( sX.length > 0 )
		{
			var ccX = sX.pop();
			var ccY = sY.pop();
			
			colorMap.set( ccX, ccY, fill );
			for( off in 0... 8 )
			{
				//PERFORANCE: No static arrays, and minimize if comparisons
				var nx = ccX;
				var ny = ccY;
				
				switch( off )
				{
					case 0:
						nx--;
						ny--;
					case 1:
						nx--;
					case 2:
						nx--;
						ny++;
					case 3:
						ny--;
					case 4:
						ny++;
					case 5:
						nx++;
						ny--;
					case 6:
						nx++;
					case 7:
						nx++;
						ny++;
				}
	 
				
				//Strangely, this is better than putting continues in the switch
				//statement on possibly invalid items (at least for size 8, 4 the continues were fine)
				if( //colorMap.validIndex(nx,ny)	//PERFORMANCE
					nx >= 0 && ny >= 0
					&& nx < colorMap.sizeX	&& ny < colorMap.sizeY
					&& ( 
						colorMap.get(nx,ny) == match1
						 || colorMap.get(nx,ny) == match2  
					) )
				{
					colorMap.set( nx, ny, fill );
					sX.push( nx );
					sY.push( ny );
					
					//virtually no performance hit by this repeated comparison
					if( shadowMap != null )
						shadowMap.set( nx, ny, shadowFill );
				}
			}
		}	//while(open)
	}
	 	
	
	
	//using these offsets may be highly inefficient, but it makes the coding easier and more versatile
	
	static public var offsets4 = [ [ -1, 0 ], [ 1, 0 ], [ 0, -1], [0,1] ];
	static public var offsets8 = [ [-1,-1], [ -1, 0 ], [-1,1], 
			[ 0, -1], [ 0, 1],
			[ 1, -1], [ 1, 0], [1,1] ];
	/**
	 * Seedfills a given matrix with the target value mathing the given input
	 *
	 * @param	colorMap	[in,out] the map to fill, seeds will be forcibly set to target
	 * @param seeds [in,out] the starting seed locations (will be cleared by output time)
	 * @param	match	[in] color to match
	 * @param fill [in] color to replace with
	 */
	static public function seedFill4<T>( colorMap : Matrix<T>, 
		seeds : FastList<MatPoint>, match : T, fill : T )
	{
		//seedFill( colorMap, seeds, match, fill, offsets4 );		
		seedFill_4( colorMap, seeds, match, fill );
	} 
	
	static public function seedFill4Shadow<T>( colorMap : Matrix<T>, 
		seeds : FastList<MatPoint>, match : T, fill : T,
		shadowMap : Matrix<T>, shadowFill : T  )
	{
		//seedFill( colorMap, seeds, match, fill, offsets4, shadowMap, shadowFill );		
		seedFill_4( colorMap, seeds, match, fill, shadowMap, shadowFill );		
	} 
	
	static public function seedFill8<T>( colorMap : Matrix<T>, 
		seeds : FastList<MatPoint>, match : T, fill : T )
	{
		//strangely in haxe mode this makes very little difference (unlike for seedFill4)
		//seedFill( colorMap, seeds, match, fill, offsets8 );		
		seedFill_8( colorMap, seeds, match, fill );		
	} 
	
	static public function seedFill28<T>( colorMap : Matrix<T>, 
		seeds : FastList<MatPoint>, match1 : T, match2 : T, fill : T )
	{
		//strangely in haxe mode this makes very little difference (unlike for seedFill4)
		//seedFill( colorMap, seeds, match, fill, offsets8 );		
		seedFill_2_8( colorMap, seeds, match1, match2, fill );		
	} 
	
	/**
	 * Traces the edges of a filled area
	 *
	 * @param colorMap	[in] map of the colors to trace
	 * @param	outMap	[in] map of the output, must not be colorMap!
	 * @param match	[in] color to match / trace edges of
	 * @param	edge	[in] color to set edge
	 */
	static public function traceEdges4<T>( colorMap : Matrix<T>,
		outMap : Matrix<T>,
		match : T, edge : T )
	{
		traceEdges( colorMap, outMap, match, edge, offsets4 );
	} 
	
	static public function traceEdges8<T>( colorMap : Matrix<T>,
		outMap : Matrix<T>,
		match : T, edge : T )
	{
		traceEdges( colorMap, outMap, match, edge, offsets8 );
	} 
	
	static private function traceEdges<T>( colorMap : Matrix<T>,
		outMap : Matrix<T>,
		match : T, edge : T,
		offsets : Array<Array<Int>> )
	{
		//simply do a simple edge detecting algorithm
		for( y in 0...colorMap.sizeY )
		{
			for( x in 0...colorMap.sizeX )
			{
				if( colorMap.get(x,y) == match )
				{
					var on = false;
					if( y == 0 || y == colorMap.sizeY-1
						|| x == 0 || x == colorMap.sizeX-1 )
						on = true;
					else 
					{	
						for( off in offsets )
							if( colorMap.get( x+off[0],y+off[1]) != match )
								on = true;
					}
						
					if( on )
						outMap.set( x, y, edge );
				}
			} //end x
		}//end y
	} 

}
