package mathx;
/* This isn't really part of hte UI package, but I was getting naming conflicts on conversion to
 * As3 with the mathx.Matrix name...
 */
 
#if flash
import flash.display.Sprite;
import flash.display.Stage;
import flash.display.BitmapData;
import flash.geom.Point;
import flash.geom.Rectangle;

/**
 * Performs collisions on specific MovieClips and sprites.  This needs to be
 * attached to a particular stage in order to work with sprites on that stage.
 *
 * This only supports a Bool hit condition, if a version is needed to judge
 * the degree of overlap then a color transform / blend mode version could
 * be done and use getColorBoundsRect to determine the size of the 
 * overlapping area.
 */
class ObjectCollider
{
	var stage : Stage;
	var alpha : Float;
	public var filters : Bool;	//public only for ObjectColliderX
	
	var objx1 : mathx.workaround.ObjectColliderX;
	var objx2 : mathx.workaround.ObjectColliderX;
	
	/**
	 * Creates a new collider for the given stage
	 *
	 * @param	astage	[in,keep] in whcih stage the comparisions are done
	 * @param	aalpha	[in] what alpha level match is considered a collision
	 * @param	afilters	[in] use filters in comparision (otherwise they are stripped for comparison)
	 */
	public function new( astage : Stage, ?aalpha : Null<Float>, ?afilters : Null<Bool> )
	{
		alpha = if( aalpha == null ) 0.5 else aalpha;
		filters = if( afilters == null ) true else afilters;
		stage = astage;
		
		objx1 = new mathx.workaround.ObjectColliderX( this );
		objx2 = new mathx.workaround.ObjectColliderX( this );
	}
	
	public function createCollisionBitmaps( s1 : Sprite, s2 : Sprite ) : { a : BitmapData, b : BitmapData }
	{
		//get the real bounds of each item
		var r1 = s1.getBounds( stage );
		var r2 = s2.getBounds( stage );
		var intr = r1.intersection( r2 );
		
		var bd1 = objx1.getCollisionBitmap( s1, intr );
		var bd2 = objx2.getCollisionBitmap( s2, intr );
		
		return { a : bd1, b: bd2 };
	}
	
	static var originPoint = new Point();
	public function checkCollision( bd1 : BitmapData, bd2 : BitmapData, alpha : Float ) : Bool
	{
		//accounts for about 15% of time in comparison, smaller cached sizes would likely help
		var a = Math.round( alpha * 0xff );
		return bd1.hitTest( originPoint, a, bd2, originPoint, a );
	}
	
	public function doCollide( s1 : Sprite, s2 : Sprite ) : Bool
	{
		//short-track checking if bounding boxes don't collide
		if( !s1.hitTestObject( s2 ) )
			return false;
			
		var bits = createCollisionBitmaps( s1, s2 );
		var ret = checkCollision( bits.a, bits.b, alpha );
		//might as well use this function...
		//bits.a.dispose();
		//bits.b.dispose();
		return ret;
	}
}

#end