package mathx;

import mathx.Matrix;
import mathx.MatPoint;
import mathx.MatPointF;
import mathx.MatrixUtil;

class Tracer
{
	var map : Matrix<Int>;
	public var traced : Matrix<Int>;
	public var traces : Array<Array<MatPoint>>;
	public var tracesColor : Array<Int>;
	public var tracesOverlap : Array<Bool>;	//does the trace overlap another trace
	
	public function new( amap : Matrix<Int> )
	{
		map = amap;
		
		traced = Matrix.create( map.sizeX, map.sizeY, 0 );
		traces = new Array<Array<MatPoint>>();
		tracesColor = new Array<Int>();
		tracesOverlap = new Array<Bool>();
		
		var tc = 1;
		for( at in traced.xOrderIter() )
		{
			if( traced.get( at.x, at.y ) != 0 )
				continue;
				
			var seeds = traceFrom( at, tc );
			var overlap = !onEdge( seeds );
			tracesOverlap.push( overlap );
			var temp = map.copy();
			MatrixUtil.seedFill4Shadow( temp, seeds, map.get(at.x,at.y), map.get(at.x,at.y)- 1, traced, tc );
			tc++;
		}
	}
	
	/**
	 * determines where any of the seeds are on the edge of the map, in which
	 * case we are assuming this trace doesn't overlap any other trace
	 *
	 * Is this always true, and is it the only case?
	 */
	function onEdge( seeds : FastList<MatPoint> ) : Bool
	{
		for( mp in seeds ) 
		{
			if( mp.x == 0 || mp.y == 0 || mp.x == traced.sizeX || mp.y == traced.sizeY )
				return true;
		}
		return false;
	}	 
	
	/**
	 * Returns a list of seeds for the fill algorithm
	 */
	function traceFrom( start : MatPoint, tc : Int ) : FastList<MatPoint>
	{
		var at = start.copy();
		var seeds = new FastList<MatPoint>();
		
		var tr = new Array<MatPoint>();
		var dir = 0;	//0 = to right, 1 = down, 2 = to left, 3 = up
		var color = map.get( at.x, at.y );
		
		//on initial scan only to the right is possible (from enclosing algorithm, xOrderIter)
		Assert.isTrue( checkDir( start, 0, color ) );
		
		tr.push( at.copy() );
		do
		{
			var sd = dir;
			while( !checkDir( at, dir, color ) )
			{
				//this closes off objects, no diagonal connections considered
				dir++;
				if( dir > 3 )
					dir = 0;
					
				//this opens objects, considering diagnols, but unforunately rescans certain
				//areas, as there is ambiguity in the source (the order of the operation changes
				//the groupings.  I would need to checked the scanned state in order to support
				//this
				/*dir--;
				if( dir < 0 )
					dir = 3;*/
					
				//this point can't be traced, all the colors are the same (inner region) (something has gone wrong)
				Assert.isTrue( dir != sd );
			}
				
			//NOTE: we don't check the validity of the set points here since it isn't possible
			//from above to trace an invalid point!
			var tm = at.copy();
			switch( dir )
			{
				case 0: 
					traced.set( at.x, at.y, tc );
					at.x++;
				case 1: 
					tm.x--;
					at.y++;
				case 2: 
					tm.x--;
					tm.y--;
					at.x--;
				case 3: 
					tm.y--;
					at.y--;
			}
			traced.set( tm.x, tm.y, tc );
			seeds.push( tm );
			
			//do not add first point to list
			if( at.equals( tr[0] ) )
				break;
			tr.push( at.copy() );
		} while( true );
		
		traces.push( tr );
		tracesColor.push( color );
		return seeds;
	}
	
	static var offsets = [
		[ [ 0, 0 ], [ 0, -1 ] ],	//right
		[ [ -1, 0 ], [ 0, 0 ] ], //down
		[ [ -1, -1 ], [ -1, 0 ] ], //left
		[ [ 0, -1 ], [ -1, -1 ] ]	//up
		];
	//color being traced is always maintained on the right side of the traversal
	function checkDir( at : MatPoint, dir : Int, color : Int ) : Bool
	{
		//trace( "" + at.x + "," + at.y + " => " + map.get( at.x, at.y ) + " / " + color + ", " + dir);
		//233 - 256
		/*switch( dir )
		{
			case 0: //right
				if( at.x == map.sizeX ||  at.y == map.sizeY )
					return false;
				return map.get( at.x,  at.y ) == color
					&& 
						( at.y == 0
						|| map.get( at.x, at.y -1 ) != color );
					
			case 1: //down
				if( at.y == map.sizeY || at.x == 0 )
					return false;
				return map.get( at.x - 1, at.y ) == color
					&& 
						( at.x == map.sizeX
							|| map.get( at.x, at.y ) != color );
			
			case 2: //left
				if( at.x == 0 || at.y == 0 )
					return false;
				return map.get( at.x - 1, at.y - 1 ) == color
					&&
						( at.y == map.sizeY
							|| map.get( at.x -1, at.y ) != color );
			
			case 3: //up
				if( at.y == 0 || at.x == map.sizeX )
					return false;
				return map.get( at.x, at.y -1 ) == color
					&&
						( at.x == 0
							|| map.get( at.x - 1, at.y -1 ) != color );
		}
		
		return false;
		*/
		
		//255 - 281 (significantly slower)
		//237 - 262 (removing temporaries -- helps so much, that is fucking sad!)
		//---I'll live with this, it isn't great, but it gives me a chance to do left-winding...
		//248 - 277 (doing more if's/flow, makes it worse)
		var off = offsets[dir];	//removing causes slowdown, so... (individual offsets didn't help)
		//var tcolor = MatPoint.at( at.x + off[0][0], at.y + off[0][1] );
		//var notColor = MatPoint.at( at.x + off[1][0], at.y + off[1][1] );
		
		return 
			map.validIndex( /*tcolor.x*/ at.x + off[0][0], /*tcolor.y*/  at.y + off[0][1] )
			&& map.get( /*tcolor.x*/  at.x + off[0][0], /*tcolor.y*/  at.y + off[0][1] ) == color
			&& 
				( !map.validIndex( /*notColor.x*/ at.x + off[1][0], /*notColor.y*/ at.y + off[1][1] )
					|| map.get( /*notColor.x*/ at.x + off[1][0], /*notColor.y*/ at.y + off[1][1] ) != color );
	}
	
}
