package mathx.test;

import flash.display.Sprite;
import flash.display.Stage;
import flash.display.Bitmap;
import mathx.ObjectCollider;
import flash.events.Event;
import flash.events.MouseEvent;
import flash.filters.DropShadowFilter;
import flash.filters.GlowFilter;

import ui.AnimationManager;

//the debugging class used while developing ObjectCollider
class Collide extends Sprite
{
	public function new( oc : ObjectCollider, s1 : Sprite, s2 : Sprite )
	{
		super();
		
		//take stage from s1, assume to be as in oc
		var r1 = s1.getBounds( s1.stage );
		graphics.lineStyle( 0, 0x800000 );
		graphics.drawRect( r1.left, r1.top, r1.width, r1.height );
		var r2 = s2.getBounds( s1.stage );
		graphics.drawRect( r2.left, r2.top, r2.width, r2.height );
		
		var intr = r1.intersection( r2 );
		graphics.lineStyle( 0, 0x008080 );
		graphics.drawRect( intr.left, intr.top, intr.width, intr.height );
		
		var bits = oc.createCollisionBitmaps( s1, s2 );
		
		var hit = oc.checkCollision( bits.a, bits.b, 0.5 );
		graphics.lineStyle( 2, if ( hit ) 0xff0000 else 0x00ff00 );
		graphics.drawRect( 0, 0, intr.width, intr.height );
		
		var bit1 = new Bitmap( bits.a );
		addChild( bit1 );
		var bit2 = new Bitmap( bits.b );
		addChild( bit2 );
	}
}

class CollisionTest extends ui.StageLayout, implements ui.Animation
{
	var move : Sprite;
	var target : Sprite;
	var show : Collide;
	var collider : ObjectCollider;
	
	var perf : Bool;
	var perfTime : Float;	//start of performance test
	var perfCount : Int;
	
	public function new()
	{
		//the initial size/shape is for performance testing
		perf  = true;	//turn on performance testing
		perfTime = 0;
		perfCount = 0;
		super();
		
		addEventListener( Event.ADDED, onAdded );
		
		var a = new Array();
		a.push( new DropShadowFilter( 20) );
		
		move = new Sprite();
		move.graphics.beginFill( 0xff8000 );
		move.graphics.drawCircle( 0, 0, 50 );
		move.graphics.drawRoundRect( 50, 10, 25, 25, 10 );
		move.graphics.endFill( );
		move.rotation = 130;
		move.filters = a;
		move.x = 260;
		move.y = 110;
		addChild( move );
		
		var b = new Array();
		b.push( new GlowFilter( 0x00ff00, 0.5, 15, 15 ) );
		target = new Sprite();
		target.x = 200;
		target.y = 0;
		target.rotation = 45;
		target.filters = b;
		addChild( target );
	}
	
	function resize( w : Float, h : Float )
	{
		graphics.clear();
		graphics.beginFill( 0x808080 );
		graphics.drawRect( 0, 0, w, h );
		graphics.endFill();
		
		target.graphics.clear();
		target.graphics.beginFill( 0x80 );
		target.graphics.drawCircle( 0, 0, w/7 );
		target.graphics.drawRect( 100, 100, w/8, 14 );
		target.graphics.endFill( );
	}
	
	function added( )
	{
		if( !perf )
			stage.addEventListener( MouseEvent.MOUSE_MOVE, onMouseMove );
		collider = new ObjectCollider( stage, 0.5, false );
		
		AnimationManager.addListener( this );
	}
	
	public function onAnimate( time : Float  ) : Bool
	{
		if( perf )
		{
			//looping here to avoid Flash redrawing overhead in tests
			//100 is overkill, but it gives a constant number
			for( i in 0...100 )
			{
				move.rotation++;
				checkHit();
			}
		}
		else
		{
			move.rotation++;
			checkHit();
		}
		
		return true;
	}
	
	function onMouseMove( evt : MouseEvent )
	{
		evt.updateAfterEvent();
		
		move.x = evt.stageX;
		move.y = evt.stageY;
		
		checkHit();
	}
	
	function checkHit()
	{
		if( perf )
		{
			var stime = flash.Lib.getTimer();
			var hit = collider.doCollide( move, target );
			var etime = flash.Lib.getTimer();
			perfTime += etime - stime;
			perfCount++;
			if( perfCount == 360 )
			{
				trace( perfTime / perfCount );	//once per revolution
				perfTime = 0;
				perfCount = 0;
			}
				
			//NOTE: The time is almost random, from 0.3 thru to 1.0 !!!
			
			//not showing the child only makes a tiny time difference (could only by GC/mem)
			return;
		}
		
		if( show != null )
		{
			removeChild( show );
			show = null;
		}
		
		if( move.hitTestObject( target ) )
		{
			show = new Collide( collider, move, target );
			addChild( show );
		}
		
	}
	
	static public function main()
	{
		ui.StageLayout.setup( new CollisionTest() );
	}
}