package mathx.test;

import mathx.Geometry;
import mathx.MatPointF;
import mathx.MatPoint;
import mathx.MathUtil;
import mathx.MatrixGeom;

class TestGeometry extends haxe.unit.TestCase
{
	private function assertEq( a : Float, b : Float )
	{
		if( MathUtil.equals( a, b ) )
		{
			assertTrue( true );	//force test framework to notice our assert (otherwise it produces a warning)
			return;
		}
		trace( "" + a + " != " + b );
		assertTrue( false );
	}
	
	private function assertEqMP( a : MatPoint, b : MatPoint )
	{
		if( a.x == b.x && a.y == b.y )
		{
			assertTrue( true );
			return;
		}
		trace( "[" + a.x + "," + a.y + "] != " +"[" + b.x + "," + b.y + "]" );
		assertTrue( false );
	}
	
	public function testBasic()
	{
		var a = MatPointF.at( 4, 3 );
		var b = MatPointF.at( 2, -2 );
		
		var c = a.sub( b );
		assertEquals( 2.0, c.x );
		assertEquals( 5.0, c.y );
		c = a.add( b );
		assertEquals( 6.0, c.x );
		assertEquals( 1.0, c.y );
		
		//xAxisAngle quadrant
		var xb =  MatPointF.at( 0, 0 );
		var x1 =  MatPointF.at( 2,2 );
		var x2 =  MatPointF.at(-2,2);
		var x3 =  MatPointF.at(-2,-2);
		var x4 =  MatPointF.at(2,-2);
		assertEq( Geometry.xAxisAngle( xb, x1 ), MathUtil.PI/4 );
		assertEq( Geometry.xAxisAngle( xb, x2 ), 3*MathUtil.PI/4 );
		assertEq( Geometry.xAxisAngle( xb, x3 ), -3*MathUtil.PI/4 );
		assertEq( Geometry.xAxisAngle( xb, x4 ), -MathUtil.PI/4 );
		
	}
	
	public function testMatrixGeom()
	{
		var at = MatPointF.at( 3.5, 7.5 );
		var dir = MatPointF.at( 1, 0 );	//horizontal
		assertEqMP( MatrixGeom.nextGridFrom( at, dir ), MatPoint.at( 4, 7 ) );
		dir.y = 1;	//diagonal
		dir = Geometry.unit( dir );
		assertEqMP( MatrixGeom.nextGridFrom( at, dir ), MatPoint.at( 4, 8 ) );
		at.x = 3.1;
		at.y = 7.1;	//diagonal, too short
		assertEqMP( MatrixGeom.nextGridFrom( at, dir ), MatPoint.at( 4, 8 ) );
		at.y = 7.9;	//diagonal, though down first
		assertEqMP( MatrixGeom.nextGridFrom( at, dir ), MatPoint.at( 3, 8 ) );
		//no motion
		assertEqMP( MatrixGeom.nextGridFrom( at, MatPointF.at( 0, 0 ) ), MatPoint.at( 3, 7 ) );
		
		for( dir in MatrixGeom.dirs4 )
			assertEqMP( MatrixGeom.nextGridBase( dir.promoteF() ), dir );
		assertEqMP( MatrixGeom.nextGridBase( MatPointF.at( 0, 0 ) ), MatPoint.at( 0, 0 ) );
	}
}
