package mathx.test;

import mathx.Matrix;
import mathx.MatrixUtil;
import mathx.ArrayUtil;
import mathx.MatPoint;

#if flash
import mathx.FlashMatrix;
typedef IntTestMatrix = FlashMatrix;

#else neko
typedef IntTestMatrix = Matrix<Int>;

#end

class TestMatrix extends haxe.unit.TestCase
{
	public function testBasic()
	{
		var q = { x : 1, y : 1 };
		var r = q;
		q.x += 1;
		trace( r );
	
		var origA : Array<Array<Int>> = [ [ 1, 2, 3], [4,5,6] ];
		var orig = IntTestMatrix.fromArrayMat(  origA );
		var n = orig.copy();
		assertEquals( orig.get( 1, 1 ), 5 );	//checking very basics
		assertTrue( MatrixUtil.equals( orig, n ) );
		n.set( 1,1, 7);
		assertEquals( orig.get(1,1), 5 );	//should not have been modified
		assertFalse( MatrixUtil.equals( orig, n ) );
		
		var q = orig.createFlip( );
		assertTrue( ArrayUtil.equals( q.extractRow(1), orig.extractRow(0) ) );
		assertTrue( ArrayUtil.equals( q.extractRow(0), orig.extractRow(1) ) );
		
		var rotA : Array<Array<Int>> = [ [ 4, 1 ], [ 5, 2], [6, 3 ] ];
		var rot = IntTestMatrix.fromArrayMat( rotA );
		var oRot = MatrixUtil.rotate90C( orig );
		assertTrue( MatrixUtil.equals( rot, oRot ) );
		
		{
			//testing iterators (the first one doesn't actually assume an order, any order would work)
			var count = IntTestMatrix.create( 10, 5, 0 );
			for( mp in count.boxOrderIter( MatPoint.at( 2, 3 ) ) )
			{
				assertTrue( count.validIndex( mp.x, mp.y ) );
				count.set( mp.x, mp.y, count.get( mp.x, mp.y ) + 1 );
			}
			
			var total = 0;
			for( mp in count.xOrderIter() )
			{
				assertEquals( 1, count.get(mp.x,mp.y) );
				total++;
			}
			
			assertEquals( total, 10 * 5 );
			
			//simple check (TODO: This assumes an order, which is not guaranteed)
			var mp = count.subIter( MatPoint.at( 2, 3 ), MatPoint.at( 3, 2 ) );
			assertEquals( 2, mp.next().x ); //2,3
			assertEquals( 3, mp.next().y ); //3,3
			assertEquals( 4, mp.next().x ); //4,3
			assertEquals( 4, mp.next().y ); //2,4
			assertEquals( 3, mp.next().x ); //3,4
			assertEquals( 4, mp.next().y ); //4,4
			
			assertTrue( !mp.hasNext() );
		}
	}
	
	public function testBorderIter()
	{
		var mat = IntTestMatrix.create( 10, 10, 0 );
		var it = mat.borderIter( MatPoint.at( 3, 4 ), MatPoint.at( 2, 3 ), 1 );
		assertEquals( 2, it.next().x );	//2,3
		assertEquals( 3, it.next().y );	//3,3
		for( i in 0...(20-2) )
		{
			var q = it.next();
			assertTrue( mat.validIndex( q.x, q.y ) );
		}
		assertFalse( it.hasNext() );
	}	
}
