package mathx.test;

import mathx.Matrix;
import mathx.MatPoint;
import mathx.MatrixUtil;

#if flash
import mathx.FlashMatrix;
#end

class TestPerf extends haxe.unit.TestCase
{
	var lastTime : Float;
	private function startT()
	{
		lastTime = 0;
#if flash
		lastTime = flash.Lib.getTimer();
#end
	}
	
	private function endT( str : String )
	{
		var nTime = lastTime;
#if flash
		nTime = flash.Lib.getTimer();
#end
		trace( str + " " + (nTime - lastTime) );
	}
	
	//FlashMatrix actually appears to be a tad bit slower...
	public function testMatrixSpeed()
	{
		var norm = Matrix.create( 50, 50, 1 );
		var total = 0;
		startT();
		//for( i in 0...10 )
			for( mp in norm.xOrderIter() )
				total += norm.get( mp.x, mp.y );
		endT( "Matrix<Int>" );
		
		//don't use since AS3 compiler has problems with it
#if flash
		norm = FlashMatrix.create( 50, 50, 1 );
		total = 0;
		startT();
		//for( i in 0...10 )
			for( mp in norm.xOrderIter() )
				total += norm.get( mp.x, mp.y );
		endT( "FlashMatrix" );
#end

		assertTrue( true );
	}
	
	static var matSize = 50;
	static var loop = 10;
	public function testIterator()
	{
		
		var norm = Matrix.create( matSize, matSize, 1 );
		var total = 0;
		startT();
		for( i in 0...loop )
			for( mp in norm.xOrderIter() )
				total++;	//don't do any access
		endT( "Iteration" );
		
		norm = Matrix.create( matSize, matSize, 1 );
		total = 0;
		startT();
		for( i in 0...loop )
			for( y in 0...matSize )
				for( x in 0...matSize )
					total+= norm.get( x, y );
		endT( "Access" );
		
		startT();
		for( i in 0...loop )
			for( y in 0...matSize )
				for( x in 0...matSize )
					total++;
		endT( "Pure Iter" );
		
		assertTrue( true );
	}
	
	public function testRawAccess()
	{
		var data : Array<Int> = new Array<Int>();
		startT();
		for( i in 0...matSize )
			for( j in 0...matSize )
				data.push( 0 );
		endT( "RawInit" );
		
		var total = 0;
		startT();
		for( i in 0...loop )
			for( y in 0...matSize )
				for( x in 0...matSize )
					total += data[ y*matSize + x ];
		endT("RawGet");	//faster than BytesGet, strange...  (a bit faster than matrix.get)
		
		var bytes = new flash.utils.ByteArray();
		bytes.length = matSize * matSize;
		startT();
		for( i in 0...loop )
			for( y in 0...matSize )
				for( x in 0...matSize )
				{
					//Direct AS3 subscripting is 20% faster
					bytes.position = y * matSize + x;
					total += bytes.readByte();
				}
		endT("BytesGet");
		
		assertTrue( true );
	}
	
	public function testBoxIterator()
	{
		var norm = Matrix.create( matSize, matSize, 1 );
		var total = 0;
		startT();
		for( i in 0...loop )
			for( mp in norm.boxOrderIter( MatPoint.at( matSize>>1, matSize>>1 ) ) )
				total++;	//don't do any access
		endT( "BoxIteration" );
		
		
		assertTrue( true );
	}
	
	public function testSeedFill()
	{
		var norm = Matrix.create( matSize, matSize, 0 );
		startT();
		for( i in 0...loop )
		{
			var lo = new FastList<MatPoint>();
			lo.push( MatPoint.at( 0, 0 ) );
			MatrixUtil.seedFill4( norm, lo, i, i+1 );
		}
		endT("SeedFill4");
		for( mp in norm.xOrderIter() )
			assertEquals( loop, norm.get( mp.x, mp.y ) );
		
		norm = Matrix.create( matSize, matSize, 0 );
		startT();
		for( i in 0...loop )
		{
			var lo = new FastList<MatPoint>();
			lo.push( MatPoint.at( 0, 0 ) );
			MatrixUtil.seedFill8( norm, lo, i, i+1 );
		}
		endT("SeedFill8");
		for( mp in norm.xOrderIter() )
			assertEquals( loop, norm.get( mp.x, mp.y ) );
		assertTrue( true );
	}
	
	public function testMatPoint()
	{
		var q;
		startT();
		for( i in 0...loop )
			for( j in 0...(matSize*matSize) )
				q = MatPoint.at( i, j );
		endT( "MatPoint create" );
		
		assertTrue( true );
	}
}
