package mathx.workaround;

import flash.display.BitmapData;
import flash.display.Sprite;
import flash.display.Stage;
import flash.display.BitmapData;
import flash.geom.Point;
import flash.geom.Rectangle;

import mathx.ObjectCollider;

/**
 * To work around the haxe =>AS3 converion error with namespaces and
 * duplicate names
 *
 * Now has added caching value, this reduces the memory use of repeated
 * calls to this object.  It however means that each object can only be used
 * for one bitmap at a time (callers of ObjectCollider need to know that too).
 *
 * TODO: It may be worth it to cache different sizes of the bitdata to improve
 * the fill and hitTest functions, how much difference this would make is
 * unknown though (and it would consume more memory...)
 * NOTE: caching makes only about a 10% little speed difference at all, only memory, strange...
 * Thus, don't bother caching extra sizes
 *
 * In the collisiontest the perf indicator is giving a value of 0.6, which I belive is ms, which
 * means optimizing here further is a waste of time.
 */
class ObjectColliderX
{
	var oc : ObjectCollider;
	var bitdata : BitmapData;
	var fullSize : Rectangle;
	
	public function new( oc : ObjectCollider )
	{
		this.oc = oc;
		fullSize = new Rectangle( 0, 0, 10, 10 );
		bitdata = new BitmapData( Math.floor(fullSize.width), Math.floor(fullSize.height), true, 0x000000 );
		
		thisSize = new Rectangle( 0, 0, 0, 0 );
	}
	
	var thisSize : Rectangle;	//keep around to use again
	public function getCollisionBitmap( s1 : Sprite, intr : Rectangle ) : BitmapData
	{
		var relR1 : flash.geom.Matrix = s1.transform.concatenatedMatrix.clone();
		relR1.tx -= intr.left;	//transform to be relative...
		relR1.ty -= intr.top;
		
		//clear filters
		var oldF = s1.filters;
		if( !oc.filters )
			s1.filters = null;
			
		//use Ceil to ensure large enough, and Max to guarantree at least one size
		thisSize.width = Math.ceil( Math.max( 1, intr.width) );
		thisSize.height = Math.ceil( Math.max( 1, intr.height) );
		if( thisSize.width > bitdata.width || thisSize.height > bitdata.height ) 
		{
			//we need a larger bitmap
			bitdata = new BitmapData( Math.floor( thisSize.width ), Math.floor( thisSize.height ), true, 0x00000 );
			fullSize.right = thisSize.width;
			fullSize.bottom = thisSize.height;
		} 
		else		//this takes about 15-20% of the time, would be improved by smaller cached items
			bitdata.fillRect( fullSize, 0x000000 );	//clean up old data
			
		//does specifying clip mode have a significant improvement here
		//see test, it looks marginally better...
		//takes (obviously a lot) 60% of the time
		bitdata.draw( s1, relR1, null/*color*/, null/*blend*/, thisSize/*clip*/ );
		
		if( !oc.filters )
			s1.filters = oldF;
		return bitdata;
	}
}
