package ui;

import flash.events.Event;

/**
 * This makes up for a lack in Haxe to be able to cast to a function signature...
 * that is, though functions can be provided as arguments, there is no way
 * that if the argument is dynamic that a function can be used
 */
interface FunctionWrapper
{
	function call() : Void;
}

class FunctionWrapper0 implements FunctionWrapper
{
	var func : Void -> Void;
	
	public function new( func : Void -> Void ) { this.func = func; }
	
	public function call() { func(); }
}

class FunctionWrapper1 implements FunctionWrapper
{
	var func : Dynamic -> Void;
	var param : Dynamic;
	
	public function new( func : Dynamic -> Void, param : Dynamic ) 
	{ 
		this.func = func; 
		this.param = param;
	}
	
	public function call() { func( param ); }
}

/**
 * Allow use  of dynamic action objects in various locations.  This handler
 * will then manage how those actions are executed.
 */
class Action
{
	/**
	 * Handles a dnyamic event
	 *
	 * @param	evtHandler [in] if event is flash event this is where it will be dispatched
	 * @param event	[in] dynamic event
	 */
	static public function handle( evtHandler : flash.events.IEventDispatcher, event : Dynamic ) 
	{
		var c;
		if( Std.is( event, MenuEvent ) )
			c = cast( event, MenuEvent ).fixClone();		//do workaround for haxe defect with cloning in MenuEvent (see there for more info)
		else if( Std.is( event, Event ) )
			c = event.clone();	
		else if( Std.is( event, FunctionWrapper ) )
			cast( event, FunctionWrapper ).call();
		else
			Assert.fail( "handle called with invalid event" );
			
		if( Std.is( c, Event ) )
		{
			Assert.notNull( evtHandler );
			evtHandler.dispatchEvent( cast( c, Event ) );
		}
	}
	
	static public function bind( func : Void -> Void ) : Dynamic
	{
		return new FunctionWrapper0( func );
	}
	
	static public function bind1( func : Dynamic -> Void, param : Dynamic ) : Dynamic
	{
		return new FunctionWrapper1( func, param );
	}
	
	/**
	 * Converts some standard known types into actions.  Known actions will
	 * be left as is.
	 */
	static public function translate( event : Dynamic ) : Dynamic
	{
		if( Std.is( event, String ) )
			event = new MenuEvent( event, null );
		else if( !Std.is( event, FunctionWrapper ) )
			Assert.is( event, Event );
		
		return event;
	}
}