package ui;

import flash.display.Sprite;
import flash.events.Event;

/**
 * A typical base class for Sprites which are animated.
 *
 * Override the _animate function in order to animate a derived class.
 * The animation will only be done while this Sprite is a child of another
 * sprite and is visible (also on the stage/in display list).  That is, this should
 * not be used if you need to always receive animation events, it is targeted
 * strictly towards controls with cosmetic animation.
 */
class AnimatedWidget extends SpriteWidget, implements Animation
{
	public function new()
	{
		super();
		
		addEventListener( Event.ADDED, onAnimAdded );
		addEventListener( Event.REMOVED, onAnimRemoved );
	}
	
	function onAnimAdded( evt : Event )
	{
		if( evt.target != this )
			return;
			
		AnimationManager.addListener( this );
	}
	
	function onAnimRemoved( evt : Event )
	{
		if( evt.target != this )
			return;
			
		AnimationManager.removeListener( this );
	}
	
	public function onAnimate( elapsed : Float ) : Bool
	{
		//don't animate if not being displayed...
		if( visible && stage != null )
			_animate( elapsed );
		
		//continue receiving events
		return true;
	}
	
	function _animate( elapsed : Float )
	{
		Assert.pureVirtual();
	}
}
