package ui;

import flash.utils.Timer;
import flash.utils.Dictionary;
import flash.events.TimerEvent;

/**
 * Managers timer threads and callbacks for animation in UI widgets
 */
class AnimationManager
{
	static var timer : Timer;
	static var listeners : Dictionary;
	static var toRemove : Array<Animation>;	
	static var forceUpdate : Bool = false;
	
	/**
	 * Initialize the AnimationManager.
	 *
	 * @param	fps	[in] the preferrred frames per second for all animation
	 */
	static public function init( fps : Int ) 
	{
		flash.Lib.current.stage.stage.frameRate = fps;
		
		timer = new Timer( 1000.0 / fps );
		timer.addEventListener( TimerEvent.TIMER, onTimer );
		lastTime = flash.Lib.getTimer();
		timer.start();
		
		inTick = false;
		//listeners = new Array<AMCallback>();
		listeners = new Dictionary( true );	//use weak references
		
		toRemove = new Array<Animation>();
	}
	
	static var inTick : Bool;
	static var lastTime : Float;
	static function onTimer( evt : TimerEvent )
	{
		inTick = true;
		
		//determine elapse time
		var ntime = flash.Lib.getTimer();
		var elapsed = (ntime - lastTime) / 1000.0;	//express in seconds always
		lastTime = ntime;
		
		//NOTE: A note from the GameDriver where this originated
		//performance is slightly higher without a forced update, just keep the framerate
		//high and this is fine!
		if( forceUpdate )
			evt.updateAfterEvent();
		
		var q : Iterator<Dynamic> = untyped (__keys__(listeners)).iterator();
		for( i in q )
		{
			if( i == null )
				continue;
				
			var x : Dynamic = untyped i;
			var anim : Animation = untyped listeners[x];
			//var anim : Animation = i;
			if( anim == null )
				continue;
				
			var rem : Bool = anim.onAnimate( elapsed );
			if( !rem )
				removeListener( anim );
		}

		inTick = false;
		
		cleanup();
	}
	
	static function traceItems()
	{
		var q : Iterator<Dynamic> = untyped (__keys__(listeners)).iterator();
		for( i in q )
			trace( i );
	}
	
	static public function pause( )
	{
		timer.stop();
	}
	
	static public function resume( )
	{
		timer.start();
	}
	
	/**
	 * Adds a listener to the timer threading.
	 *
	 * @param	anim	[in]Animation element to handle the events
	 * @param	timeout	[in] preferred time elapsed between events
	 */
	static public function addListener( anim : Animation, ?timeout : Null<Int> )
	{
		//TODO: timeout complete ignored now...
		untyped listeners[anim] = anim;
	}
	
	static public function removeListener( anim : Animation )
	{
		toRemove.push( anim );
		if( !inTick )
			cleanup();
	}
	
	static function cleanup()
	{
		if( toRemove.length == 0 )
			return;
			
		while( true ) 
		{
			var anim = toRemove.shift();
			if( anim == null )
				break;
			untyped __delete__(listeners,anim);
		}
	}
}
