package ui;

import flash.display.BitmapData;
import flash.display.Graphics;

/**
 * NOTE: all the #if as3 nonsense is to ensure this works when converted to AS3 as well as
 * compiled directly in Haxe.  It is a bug in haxe arising from how these Enum'd String types
 * are handled.  Users of this class can simply use the Enum'd types as expected...
 */
enum BrushType
{
	None;
	Solid( color : Int, alpha : Float );
	Bitmap( bd : BitmapData );
#if as3
	Gradient( type : String, colors : Array<UInt>, alphas : Array<Dynamic>, ratios : Array<Dynamic>, 
		matrix :	flash.geom.Matrix, spreadMethod : String, interpolationMethod : String, focalPointRatio : Float );
#else true
	Gradient( type : flash.display.GradientType, colors : Array<UInt>, alphas : Array<Dynamic>, ratios : Array<Dynamic>, 
		matrix :	flash.geom.Matrix, spreadMethod : flash.display.SpreadMethod, interpolationMethod : flash.display.InterpolationMethod, focalPointRatio : Float );
#end
}

/**
 * Creates a logical brush which can be used for filling drawn objects.
 */
class Brush
{
	var type : BrushType;
	
	private function new( bt : BrushType )
	{
		type = bt;
	}
	
	/**
	 * Determines where there is a fill or not
	 */
	public function isNonEmpty() : Bool
	{
		return type != None;
	}
	
	/**
	 * Applies the brush to the graphics object, will be used
	 * to draw the next objects
	 */
	public function apply( graphics : Graphics )
	{
		switch( type )
		{
			case None:
			case Solid( color, alpha ):
				graphics.beginFill( color, alpha );
			case Bitmap( bd ):
				graphics.beginBitmapFill( bd );
			case Gradient( type, colors, alphas, ratios, matrix, spreadMethod, interpolationMethod, focalPointRatio ):
				graphics.beginGradientFill( untyped type, 
					colors, alphas, ratios, matrix, 
					untyped spreadMethod, 
					untyped interpolationMethod, 
					focalPointRatio );
		}
	}
	
	static public function empty() : Brush
	{
		return new Brush( BrushType.None );
	}
	
	static public function solid( color : Int, ?alpha : Float ) : Brush
	{
		if( alpha == null )
			alpha = 1;
			
		return new Brush( BrushType.Solid( color, alpha ) );
	}
	
	static public function bitmap( bd : BitmapData ) : Brush
	{
		return new Brush( BrushType.Bitmap( bd ) );
	}
	
#if as3
	static public function gradient( type : Dynamic, 
		colors : Array<UInt>, alphas : Array<Dynamic>, ratios : Array<Dynamic>, 
		?matrix : flash.geom.Matrix, 
		?spreadMethod : Dynamic,
		?interpolationMethod : Dynamic,
		?focalPointRatio : Float ) : Brush
#else true
	static public function gradient( type : flash.display.GradientType, 
		colors : Array<UInt>, alphas : Array<Dynamic>, ratios : Array<Dynamic>, 
		?matrix : flash.geom.Matrix, 
		?spreadMethod : flash.display.SpreadMethod,
		?interpolationMethod : flash.display.InterpolationMethod,
		?focalPointRatio : Float ) : Brush
#end
	{
		if( spreadMethod == null )
#if as3
			spreadMethod = "PAD";
#else
			spreadMethod = flash.display.SpreadMethod.PAD;
#end
		if( interpolationMethod == null )
#if as3
			interpolationMethod = "RGB";
#else		
			interpolationMethod = flash.display.InterpolationMethod.RGB;
#end
		if( focalPointRatio == null )
			focalPointRatio = 0;
			
		return new Brush( BrushType.Gradient( untyped type, colors, alphas, ratios, matrix, untyped spreadMethod, untyped interpolationMethod, focalPointRatio ) );
	}
}
