package ui;

import flash.display.Sprite;
import flash.events.MouseEvent;

import flash.filters.BitmapFilter;
import flash.filters.BevelFilter;
import flash.filters.ColorMatrixFilter;

class Button extends Sprite, implements Widget
{
	var down : Bool;
	var tf : StaticText;
		
	var filtersUp : Array<BitmapFilter>;
	var filtersDown : Array<BitmapFilter>;
	var filtersDisabled : Array<BitmapFilter>;
	
	var bevelSize : Float;
	
	var event : Dynamic;
	var color : Int;
	
	var flat : Bool;
	var transparent : Bool;
	
	var image : Widget;
	
	public function new( text : String, event : Dynamic, ?color : Null<Int> )
	{
		super();
		MixinWidget();
		
		enabled = true;
		flat = false;
		transparent = false;
		down = false;
		if( color == null )
			color = 0x8080A0;
		this.color = color;
		this.event = Action.translate( event );
		
		bevelSize = 2;
		filtersUp = new Array();
		filtersUp.push( new BevelFilter( bevelSize, 45) );
		filtersDown = new Array();
		var bf = new BevelFilter( bevelSize, 180 + 45);
		bf.highlightAlpha = 0;
		filtersDown.push( bf );
		
		if( text != null )
		{
			tf = new StaticText( text );
			tf.x = bevelSize;
			tf.y = bevelSize;
			addChild( tf );
		}
		
		addEventListener( MouseEvent.MOUSE_DOWN, onMouseDown, false, 0, true /*weak*/ );
		addEventListener( MouseEvent.MOUSE_UP, onMouseUp,false, 0, true /*weak*/ );
		addEventListener( MouseEvent.MOUSE_OVER, onMouseOver,false, 0, true /*weak*/ );
		addEventListener( MouseEvent.MOUSE_OUT, onMouseOut,false, 0, true /*weak*/ );
	}
	
	//TODO: Create setup properties/flags somehow
	public function setImage( image : Widget )
	{
		if( this.image != null )
		{
			removeChild( this.image.getNative() );
			this.image = null;
		}
		this.image = image;
		if( this.image != null )
			addChild( this.image.getNative() );
		update();			
	}
	
	public function setFlat( )
	{
		flat = true;
		update();
	}
	
	public function setTransparent()
	{
		transparent = true;
		update();
	}
			
	function onMouseDown( evt : MouseEvent )
	{
		if( !enabled )
			return;
			
		down = true;
		update();
	}
	
	function onMouseUp( evt : MouseEvent )
	{
		if( !enabled )
			return;
			
		down = false;
		update();
		
		Action.handle( this, event );
	}
	
	function onMouseOver( evt : MouseEvent )
	{
		if( !enabled )
			return;
			
		down = evt.buttonDown;
		update();
	}
	
	function onMouseOut( evt : MouseEvent )
	{
		if( !enabled )
			return;
			
		down = false;
		update();
	}
	
	function _resize( w : Float, h : Float )
	{
		//setup in case disabled
		if( !enabled && filtersDisabled == null )
		{
			filtersDisabled = new Array();
			var matrix = new Array<Float>();
    	    matrix = matrix.concat([1.0/3, 1.0/3, 1.0/3, 0, 0]); // red
        	matrix = matrix.concat([1.0/3, 1.0/3, 1.0/3, 0, 0]); // green
        	matrix = matrix.concat([1.0/3, 1.0/3, 1.0/3, 0, 0]); // blue
        	matrix = matrix.concat([0, 0, 0, 1.0, 0]); // alpha
        	filtersDisabled.push( new ColorMatrixFilter( matrix ) );
		}
		
		var pad = bevelSize;
		if( flat )
			pad = 0;
		
		if( enabled && down )
		{
			if( tf != null )
			{
				tf.x = pad *2;
				tf.y = pad * 2;
			}
			if( image != null )
				image.move( pad *2, pad * 2 );
		}
		else
		{
			if( tf != null )
			{
				tf.x = pad;
				tf.y = pad;
			}
			if( image != null )
				image.move( pad, pad );
		}
		if( tf != null )
			tf.resize( w - pad*2, h - pad*2 );
		if( image != null )
			image.resize( w - pad*2, h - pad*2 );
		
		graphics.clear();
		//graphics.beginFill( if( down ) ColorUtil.complement( color ) else color );
		//always use a background, even when transparent, to ensure clicking in button works
		//clicking is only noticed on actual content... (even if it is transparent)
		graphics.beginFill( color, transparent ? 0 : 1 );	
		graphics.drawRect( 0,0,w,h );
		graphics.endFill();
		
		if( enabled )
			filters = if( down ) filtersDown 
				else if( flat ) null
				else filtersUp;
		else
			filters = filtersDisabled;
	}
	
	public function getPrefWidth() : SizeType
	{
		return SizeType.Sum( tf.getPrefWidth(), SizeType.Fixed( 2 * bevelSize ) );
	}
	
	public function getPrefHeight() : SizeType
	{
		return SizeType.Sum( tf.getPrefHeight(), SizeType.Fixed( 2 * bevelSize ) );
	}

	
	/**
 * Provides the functionality to create a Widget of a Flash Sprite.
 */	
 
	var sizeW : Float;	//our desired size (may not be the same as width)
	var sizeH : Float;
	var enabled : Bool;
		
	/**
	 * Serves as a Ctor and must be called by the including class during its
	 * own construction.
	 */
	function MixinWidget()
	{
		sizeW = 0;
		sizeH = 0;
		enabled = true;
	}
	
	public function move( x : Float, y : Float ) : Void
	{
		this.x = x;
		this.y = y;
	}
	
	//Note that users of this mixin override the _resize function, not "resize" directly...
	public function resize( w : Float, h : Float ) : Void
	{
		sizeW = w;
		sizeH = h;
		_resize( w, h );
	}
	
	public function update( ) : Void
	{
		_resize( sizeW, sizeH );
	}
	
	public function enable( set : Bool ) : Void
	{
		enabled = set;
		update();
	}

	public function getNative() : flash.display.DisplayObjectContainer
	{
		return this;
	}
	
	
}