package ui;

import flash.display.Sprite;
import flash.events.MouseEvent;

import flash.filters.BitmapFilter;
import flash.filters.BevelFilter;

class CheckBox extends Sprite, implements Widget, implements Selectable
{
	var checked : Bool;	//state of checkbox
	var down : Bool;	//stage of mouse
	var tf : StaticText;
		
	var filtersUp : Array<BitmapFilter>;
	var filtersDown : Array<BitmapFilter>;
	
	var bevelSize : Float;
	var box : Sprite;
	
	var color : Int;
	var circle : Bool;
	
	public function new( text : String, ?color : Null<Int> )
	{
		super();
		MixinWidget();
		
		down = false;
		circle = false;
		checked = false;
		if( color == null )
			color = 0x8080A0;
		this.color = color;
		
		bevelSize = 2;
		filtersUp = new Array();
		filtersUp.push( new BevelFilter( bevelSize, 45) );
		filtersDown = new Array();
		var bf = new BevelFilter( bevelSize, 180 + 45);
		//bf.highlightAlpha = 0;	//Unlike a button we need to keep this obrder otherwise it is hard to see
		filtersDown.push( bf );
		
		tf = new StaticText( text, StaticTextAlign.Left );
		addChild( tf );
		
		//use a separate box for the box to avoid applying filter to text
		box = new Sprite();
		addChild( box );
		
		addEventListener( MouseEvent.MOUSE_DOWN, onMouseDown,false, 0, true /*weak*/ );
		addEventListener( MouseEvent.MOUSE_UP, onMouseUp,false, 0, true /*weak*/ );
		addEventListener( MouseEvent.MOUSE_OVER, onMouseOver,false, 0, true /*weak*/ );
		addEventListener( MouseEvent.MOUSE_OUT, onMouseOut,false, 0, true /*weak*/ );
	}
	
	function onMouseDown( evt : MouseEvent )
	{
		down = true;
		update();
	}
	
	function onMouseUp( evt : MouseEvent )
	{
		down = false;
		setSelected( !checked );
	}
	
	function onMouseOver( evt : MouseEvent )
	{
		down = evt.buttonDown;
		update();
	}
	
	function onMouseOut( evt : MouseEvent )
	{
		down = false;
		update();
	}
	
	function _resize( w : Float, h : Float )
	{
		var ds : Bool = Lang.BoolXOR( down, checked );
		
		tf.x = h + bevelSize;
		tf.y = 0;
		tf.resize( w - tf.x, h );
		
		box.graphics.clear();
		box.graphics.beginFill( if(ds) ColorUtil.darken( color ) else color );
		if( circle )
			box.graphics.drawEllipse( h/5,h/5,3*h/5,3*h/5 );
		else
			box.graphics.drawRect( h/5,h/5,3*h/5,3*h/5 );
		box.graphics.endFill();
		
		box.filters = if( ds ) filtersDown else filtersUp;
	}
	
	public function getSelected() : Bool
	{
		return checked;
	}
	
	public function setSelected( sel : Bool )
	{
		if( sel != checked )
		{
			checked = sel;
			update();
			if( onSelectChange != null )
				Action.handle( this, onSelectChange );
		}
	}
	
	var onSelectChange : Dynamic;
	public function setOnSelectChange( action : Dynamic )
	{
		onSelectChange = action;
	}

	public function getPrefWidth() : SizeType
	{
		return SizeType.Sum( 
			SizeType.XWidth( 1 ),	//actual size is Ratio(1), but we can't use variable amounts in Pref size
			SizeType.Sum( 
				SizeType.Fixed( bevelSize ), 
				tf.getPrefWidth() 
				)
			);
	}
	
	public function getPrefHeight() : SizeType
	{
		return tf.getPrefHeight();
	}

	
	/**
 * Provides the functionality to create a Widget of a Flash Sprite.
 */	
 
	var sizeW : Float;	//our desired size (may not be the same as width)
	var sizeH : Float;
	var enabled : Bool;
		
	/**
	 * Serves as a Ctor and must be called by the including class during its
	 * own construction.
	 */
	function MixinWidget()
	{
		sizeW = 0;
		sizeH = 0;
		enabled = true;
	}
	
	public function move( x : Float, y : Float ) : Void
	{
		this.x = x;
		this.y = y;
	}
	
	//Note that users of this mixin override the _resize function, not "resize" directly...
	public function resize( w : Float, h : Float ) : Void
	{
		sizeW = w;
		sizeH = h;
		_resize( w, h );
	}
	
	public function update( ) : Void
	{
		_resize( sizeW, sizeH );
	}
	
	public function enable( set : Bool ) : Void
	{
		enabled = set;
		update();
	}

	public function getNative() : flash.display.DisplayObjectContainer
	{
		return this;
	}
	
	
}