package ui;

import flash.display.Sprite;
import flash.display.DisplayObjectContainer;
import flash.display.DisplayObject;

/**
 * Implements a standard dialog class.  This is meant to be subclasses to create
 * the real dialog items.
 */
class Dialog extends Sprite, implements Window, implements Widget
{
	var mainLayout : LinearLayout;
	var actionLayout : LinearLayout;
	var mainBody : Widget;
	
	var fillColor : Int;
	var rounding : Int;
	
	private function new( title : String, body : Widget, ?color : Null<Int> )
	{
		super();
		MixinWidget();
		
		mainBody = body;
		
		this.fillColor = if( color == null ) 0x8080A0 else color;
		rounding = 5;
		
		mainLayout = new LinearLayout( LayoutDirection.Vertical );
		mainLayout.collapseMargin = true;
		var tf = new StaticText( title );
		tf.bold = true;
		tf.fillColor = ColorUtil.brighten( fillColor );
		
		mainLayout.add( tf, ui.SizeType.FixedFunc1( FontManager.getDefault().byLines, 1.1  ) );
		mainLayout.add( body, SizeType.Fill( 0 ) );
		addChild( mainLayout );
		
		actionLayout = new LinearLayout( LayoutDirection.Horizontal );
		actionLayout.collapseMargin = true;
		actionLayout.add( new EmptyWidget(), SizeType.Fill( 1 ) );
		mainLayout.add( actionLayout, ui.SizeType.FixedFunc1( FontManager.getDefault().byLines, 1.2 ) );
		
		//give ourselves a drop-filter
		var f = new Array();
		f.push( new flash.filters.DropShadowFilter() );
		filters = f;
	}
	
	public function addAction( label : String, action : Dynamic )
	{
		//TODO: Take the maximum of all button sizes?
		
		//the sizing now assumes the labels never change...
		var but = new Button( label, Action.bind1( onAction, action ) );
		actionLayout.add( but, ui.SizeType.Max( but.getPrefWidth(), ui.SizeType.XWidth(8) ) );
	}
	
	function onAction( action : String )
	{
		actionFinish = action;
		if( closeListener != null )
			Action.handle( this, closeListener );
	}
	
	function _resize( w : Float, h : Float )
	{
		graphics.clear();
		graphics.beginFill( fillColor );
		graphics.lineStyle( 1, 0 );
		graphics.drawRoundRect( 0, 0, w, h, 4*rounding );
		
		mainLayout.move( rounding, rounding );
		mainLayout.resize( w- 2*rounding, h- 2*rounding );
	}

	//with which action did we finish...?
	public var actionFinish : Dynamic;
	public function exec( parent : DisplayObjectContainer ) : Void
	{
		//not too much (this is far simpler than a Popup)
		actionFinish = null;
		update();
	}
	
	var closeListener : Dynamic;
	public function setCloseListener( action : Dynamic ) : Void
	{
		closeListener = action;
	}
	
	//handled by MixinWidget
	/*public function getNative() : DisplayObjectContainer
	{
		return this;
	}*/
	
	/**
	 * Resizes the dialog to be the size as specified by the Preferred
	 * size functions.  This fixates those sizes against a speicfic
	 * font/overall size.
	 *
	 * This uses the stage as the relative sizing item.
	 */
	public function usePrefSize( base : DisplayObject )
	{
		var pw = getPrefWidth();
		var sz = new ui.Sizer( 0, true, false );
		var sw = sz.fixSize( pw, base.width, base.height, 0, 0 );
		
		var ph = getPrefHeight();
		sz = new ui.Sizer( 0, true, true );
		var sh = sz.fixSize( ph, base.height, base.width, 0, 0 );
		
		move( base.x + base.width / 2 - sw/2, base.y + base.height /2 - sh/2 );
		resize( sw, sh );	
	}
	
	public function getPrefWidth() : ui.SizeType
	{
		return SizeType.Sum(
			SizeType.Fixed( 2 * rounding ), 	//for borders
			mainLayout.getPrefWidth()
			);
	}
	
	public function getPrefHeight() : ui.SizeType
	{
		return 
			SizeType.Sum(
				SizeType.Fixed( 2 * rounding ),	//for borders
				SizeType.Sum( mainLayout.getPrefHeight(), mainBody.getPrefHeight() )
				);
	}
	
	
	/**
 * Provides the functionality to create a Widget of a Flash Sprite.
 */	
 
	var sizeW : Float;	//our desired size (may not be the same as width)
	var sizeH : Float;
	var enabled : Bool;
		
	/**
	 * Serves as a Ctor and must be called by the including class during its
	 * own construction.
	 */
	function MixinWidget()
	{
		sizeW = 0;
		sizeH = 0;
		enabled = true;
	}
	
	public function move( x : Float, y : Float ) : Void
	{
		this.x = x;
		this.y = y;
	}
	
	//Note that users of this mixin override the _resize function, not "resize" directly...
	public function resize( w : Float, h : Float ) : Void
	{
		sizeW = w;
		sizeH = h;
		_resize( w, h );
	}
	
	public function update( ) : Void
	{
		_resize( sizeW, sizeH );
	}
	
	public function enable( set : Bool ) : Void
	{
		enabled = set;
		update();
	}

	public function getNative() : flash.display.DisplayObjectContainer
	{
		return this;
	}
	
	
}