package ui;

import flash.display.Sprite;

typedef DrawFunc = flash.display.Graphics -> Float -> Float  -> Void;

/**
 * Is a quick wrapper for when a widget is needed that draws something
 * but not other special handling is required.  The DrawBox accepts
 * a function to its constructor which does the actual drawing.
 */
class DrawBox extends Sprite, implements Widget
{
	var drawFunc : DrawFunc;
	
	public function new( func : DrawFunc )
	{
		super();
		MixinWidget();
		
		drawFunc = func;
	}
	
	public function setDrawFunc( func : DrawFunc )
	{
		drawFunc = func;
		update();
	}
	
	public function _resize( w : Float, h : Float )
	{
		graphics.clear();
		drawFunc( graphics, w, h );
	}
	
	/**
 * Provides the functionality to create a Widget of a Flash Sprite.
 */	
 
	var sizeW : Float;	//our desired size (may not be the same as width)
	var sizeH : Float;
	var enabled : Bool;
		
	/**
	 * Serves as a Ctor and must be called by the including class during its
	 * own construction.
	 */
	function MixinWidget()
	{
		sizeW = 0;
		sizeH = 0;
		enabled = true;
	}
	
	public function move( x : Float, y : Float ) : Void
	{
		this.x = x;
		this.y = y;
	}
	
	//Note that users of this mixin override the _resize function, not "resize" directly...
	public function resize( w : Float, h : Float ) : Void
	{
		sizeW = w;
		sizeH = h;
		_resize( w, h );
	}
	
	public function update( ) : Void
	{
		_resize( sizeW, sizeH );
	}
	
	public function enable( set : Bool ) : Void
	{
		enabled = set;
		update();
	}

	public function getNative() : flash.display.DisplayObjectContainer
	{
		return this;
	}
	

	public function getPrefWidth() : ui.SizeType
	{
		return null;
	}
	
	public function getPrefHeight() : ui.SizeType
	{
		return null;
	}
	
}
