package ui;

import flash.events.Event;

import flash.text.TextField;
import flash.text.TextFormat;

/**
 * Manages fonts for use in the display, right now it only manages one font, the default
 */
class FontManager
{

	//////////////////////////////////////////////////////////
	// Static functions for primary interaction
	
	static var rows : Int;
	static var cols : Int;
	
	static var fDefault : FontManager;
	
	/** 
	 * Initialize font manager with the minimum number of rows and cols for the
	 * flash text interface
	 * See resize
	 */
	static public function init( acols : Int, arows : Int )
	{
		rows = arows;
		cols = acols;
		fDefault = new FontManager();
			
		flash.Lib.current.stage.addEventListener( Event.RESIZE, onResize, false, 1000 );	//must be done first to ensure everything works
	}
	
	static private function onResize( evt : Dynamic )
	{
		fDefault.resize( flash.Lib.current.stage.stageWidth, flash.Lib.current.stage.stageHeight );	
	}
	
	static public function getDefault() : FontManager
	{
		return fDefault;
	}
	
	///////////////////////////////////////////////////////
	// Instances of the FontManager
	var lineHeight : Float;
	var xWidth : Float;
	var p2p : Float;	//recorded ratio from pixel size to height
	var fontName : String;
			
	static var gutter : Float = 2;
	
	private function new()
	{
		//setups the default font
		var tf = new TextField();
		tf.text = "X,w^";	//it reallys makes no difference, so long as it isn't empty!
		
		//calibrate on a large size to avoid pixel-based anomalies
		var fmt = new TextFormat();
		fmt.size = 100;
		tf.setTextFormat( fmt );
		
		var lm = tf.getLineMetrics(0);
		//Case is here for AS3 conversion error, "size" is actually of type Object, though should be an Int (this is since it may be null)
		p2p = cast( fmt.size, Int ) / (lm.ascent + lm.descent + gutter );
		
		fontName = tf.getTextFormat().font;
	}
	
	public function getFontName() : String
	{
		return fontName;
	}
	
	/**
	 * See StaticText for full use, but basically flash sticks in a non-removable
	 * gutter to the text fields, and this is always a fixed size! It is very unusual
	 * to work with text in flash.
	 */
	public function getGutter() : Float
	{
		return gutter;
	}
	
	public function fontSizeForHeight( height : Float )
	{
		return p2p * height;		
	}
	
	/**
	 * Resize will attempt adjust the number of rows and columns based on the
	 * actual screen ratio.  The rows/cols in the constructor will thus both be
	 * treated as a minimum and one of them expanded to maintain a target
	 * cell ratio.  This is hardcoded now to match a typical font.
	 */
	function resize( width : Float, height : Float )
	{
		var coreRatio = 50 / 80;	//rows are two x high (TODO: should really be based on actual font)
		var stageRatio = (width/cols) / (height/rows);
		
		if( coreRatio > stageRatio )	//screen is narrow
		{
			var targetRows = Math.floor( coreRatio * height * cols / width );
			lineHeight = height / targetRows;
			xWidth = width / cols;	
		}
		else	//screen is wide
		{
			var targetCols = Math.floor( width * rows / height / coreRatio );
			lineHeight = height / rows;
			xWidth = width / targetCols;	
		}
	}
	
	public function byLines( num : Float ) : Float
	{
		return lineHeight * num;
	}
	
	public function byXWidth( num : Float ) : Float
	{
		return xWidth * num;
	}
	
}
