package ui;

class GraphicsUtil
{
	/**
	 * Draws an approximate arc of a circle.  The arc segment may be
	 * in either direction and of any size (though no less than a dot will
	 * be drawn and no more than a circle)
	 *
	 * @param	graphics	[in,out] where to draw
	 * @param	cx,cy	[in] center of the elliptic arc
	 * @param	rx,ry	[in] radius of the ellipse
	 * @param	ast,ae	[in] start and end arc position
	 * @param	wedge	[in] true to draw a wedge, false for an arc
	 */
	static public function drawArc( graphics : flash.display.Graphics,
		cx : Float, cy : Float, 
		rx : Float, ry : Float,
		ast : Float, ae : Float,
		wedge : Bool )
	{
		var angDiff = ae - ast;
		if( angDiff == 0 )
			angDiff = Math.PI/1000;	//a minimal curve in all cases (a dot)
		if( angDiff > (2*Math.PI) )
			angDiff = 2*Math.PI;	//do not overdraw
		
		//only about 8 segments are needed to do a full convincing circle
		//so here calculate an amount based on the curve size, and use
		//10 just for extra prettiness :)
		var segments = Math.ceil( Math.abs(angDiff) / (2*Math.PI) * 10 );
		
		//for tangents to the curves
		var tan = Math.tan( angDiff / segments / 2 );	
	
		var sx = Math.cos( ast ) * rx + cx;
		var sy = Math.sin( ast ) * ry + cy;
		if( wedge )
		{
			graphics.moveTo( cx, cy );
			graphics.lineTo( sx, sy );
		}
		else
			graphics.moveTo( sx, sy );
		
		//use 8 segments... (optimize based on length of curve...)
		var angDelta = angDiff / segments ;
		for( ndx in 1...segments+1 )
		{
			var ang = ast + angDelta * ndx;
			
   			var endx = Math.cos( ang ) * rx;
   			var endy = Math.sin( ang ) * ry;
   			
   			//control point (tangents to both endpoints)
   			var ctlx = endx + rx * tan * Math.cos( ang - Math.PI/2 );
   			var ctly = endy + ry * tan * Math.sin( ang - Math.PI/2 );
   			graphics.curveTo( ctlx + cx, ctly + cy, endx + cx, endy + cy);
		}		
		
		if( wedge )
			graphics.lineTo( cx, cy );
	}

}
