package ui;

import mathx.Matrix;

// Used by GridLayout to hold items
//TODO: Could be somehow merged with that of LinearLayout?
class GridItemHolder implements Sizeable
{
	public var size : SizeType;
	public var visible : Bool;
	
	public function new( size : SizeType )
	{
		this.size = size;
		this.visible = true;
	}
	
	public function getSizeType() : SizeType
	{
		if( !visible )
			return SizeType.Hidden;
		return size;
	}
	
	public var pOffset : Float;
	public var pSize : Float;
	public function setPlacement( offset : Float, size : Float ) : Void
	{
		pOffset = offset;
		pSize = size;
	}
}

/**
 * Creates a table like layout of rows and columns.
 *
 * Update: As with other layouts, any modifications will require a manual
 * call to the "update" function.
 */
class GridLayout extends flash.display.Sprite, implements Widget
{
	var items : Matrix<Widget>;
	var cols : Array<GridItemHolder>;
	var rows : Array<GridItemHolder>;
	public var spacing : Float;
	public var collapseMargin : Bool;
	
	/**
	 * Creates a new grid layout of the given size.  The row
	 * and column weightings will all default to 1.
	 */
	public function new( ncols : Int, nrows : Int ) {
		super();
		
		spacing = 2;
		collapseMargin = false;
		items = Matrix.create( ncols, nrows, null );
		
		//assign a weight of 1 to all items
		cols = new Array<GridItemHolder>();
		for( i in 0...ncols )
			cols.push( new GridItemHolder( SizeType.Fill( 1 ) ) );
		rows = new Array<GridItemHolder>();
		for( i in 0...nrows )
			rows.push( new GridItemHolder( SizeType.Fill( 1 )  ) );
	}
	
	public function setColSize( col : Int, st : SizeType )
	{
		Assert.isTrue( col < cols.length );
		cols[col].size = st;
	}
	
	public function setRowSize( row : Int, st : SizeType )
	{
		Assert.isTrue( row < rows.length );
		rows[row].size = st;
	}
	
	/**
	 * Sets the item at the given grid location. If an item
	 * already exists there it will be replaced.
	 * Locations which have no items will be left empty
	 * during display.
	 */
	public function setItem( x : Int, y : Int, item : Widget ) 
	{
		var cur = items.get( x, y );
		if( cur != null )
			removeChild( cur.getNative() );
		
		items.set( x, y, item );
		addChild( item.getNative() );
	}
	
	public function _resize( w : Float, h : Float ) {
		var sizerC = new Sizer( spacing, collapseMargin, false );
		sizerC.size( w, h, cols );
		var sizerR = new Sizer( spacing, collapseMargin, true );
		sizerR.size( h, w, rows );
		
		for( mp in items.xOrderIter() )
		{
			var item = items.get( mp.x, mp.y );
			if( item == null )
				continue;
				
			item.move( cols[mp.x].pOffset, rows[mp.y].pOffset );
			item.resize( cols[mp.x].pSize, rows[mp.y].pSize );
		}
	}
	
	public function getPrefWidth() : ui.SizeType
	{
		//TODO: sizing based on contents?
		var sizer = new Sizer( spacing, collapseMargin, false );
		return sizer.recommend( cols );
	}
	
	public function getPrefHeight() : ui.SizeType
	{
		var sizer = new Sizer( spacing, collapseMargin, true );
		return sizer.recommend( rows );
	}
	
	
	/**
 * Provides the functionality to create a Widget of a Flash Sprite.
 */	
 
	var sizeW : Float;	//our desired size (may not be the same as width)
	var sizeH : Float;
	var enabled : Bool;
		
	/**
	 * Serves as a Ctor and must be called by the including class during its
	 * own construction.
	 */
	function MixinWidget()
	{
		sizeW = 0;
		sizeH = 0;
		enabled = true;
	}
	
	public function move( x : Float, y : Float ) : Void
	{
		this.x = x;
		this.y = y;
	}
	
	//Note that users of this mixin override the _resize function, not "resize" directly...
	public function resize( w : Float, h : Float ) : Void
	{
		sizeW = w;
		sizeH = h;
		_resize( w, h );
	}
	
	public function update( ) : Void
	{
		_resize( sizeW, sizeH );
	}
	
	public function enable( set : Bool ) : Void
	{
		enabled = set;
		update();
	}

	public function getNative() : flash.display.DisplayObjectContainer
	{
		return this;
	}
	
	
}