package ui;

/**
 * Implements a radio group of Selectable items, such that only one may
 * be selected at any given time.  This itself is not a control, but simply a 
 * behaviour for other controls.
 */
class RadioGroup
{
	var selectables : Array<Selectable>;
	var allowNone : Bool;
	
	public function new() 
	{
		selectables = new Array();
		allowNone = false;
	}
	
	public function manage( sel : Selectable )
	{
		selectables.push( sel );
		sel.setOnSelectChange( Action.bind1( onSelectChange, sel ) );
	}
	
	function onSelectChange( sel : Selectable )
	{
		if( !sel.getSelected() )
		{
			if( allowNone )
				return;
			//if none are selected then this one must be restored
			for( s in selectables )
				if( s.getSelected() )
					return;
			sel.setSelected( true );
			return;
		} else {
			for( s in selectables )
				if( s != sel )
					s.setSelected( false );
		}
	}
}
