package ui;

import flash.display.Sprite;
import flash.display.DisplayObject;
import flash.display.BitmapData;

class SimpleBox extends Sprite, implements Widget
{
	var brush : Brush;
	var pen : Pen;
	var image : DisplayObject;
	
	/**
	 * Take advantage of Haxe optional parameters in that you can specify
	 * only the ones you want without needing to worry about position --
	 * the type of the object will determine to which parameter it applies
	 */
	public function new( ?brush : Brush, ?pen : Pen, ?image : DisplayObject )
	{
		super();
		MixinWidget();
		
		this.brush = brush == null ? Brush.empty() : brush;
		this.pen = pen == null ? Pen.empty() : pen;
		this.image = image;
		
		if( this.image != null )
		{
			this.image.x = 0;
			this.image.y = 0;
			addChild( this.image );
		}
	}
	
	public function _resize( w : Float, h : Float )
	{ 
		graphics.clear();
		
		brush.apply( graphics );
		pen.apply( graphics );
		
		if( brush.isNonEmpty() || pen.isNonEmpty() )
			graphics.drawRect( 0, 0, w, h );

		graphics.endFill();
			
		if( image != null )
		{
			image.width = w;
			image.height = h;
		}
	}
	
	/**
 * Provides the functionality to create a Widget of a Flash Sprite.
 */	
 
	var sizeW : Float;	//our desired size (may not be the same as width)
	var sizeH : Float;
	var enabled : Bool;
		
	/**
	 * Serves as a Ctor and must be called by the including class during its
	 * own construction.
	 */
	function MixinWidget()
	{
		sizeW = 0;
		sizeH = 0;
		enabled = true;
	}
	
	public function move( x : Float, y : Float ) : Void
	{
		this.x = x;
		this.y = y;
	}
	
	//Note that users of this mixin override the _resize function, not "resize" directly...
	public function resize( w : Float, h : Float ) : Void
	{
		sizeW = w;
		sizeH = h;
		_resize( w, h );
	}
	
	public function update( ) : Void
	{
		_resize( sizeW, sizeH );
	}
	
	public function enable( set : Bool ) : Void
	{
		enabled = set;
		update();
	}

	public function getNative() : flash.display.DisplayObjectContainer
	{
		return this;
	}
	

	public function getPrefWidth() : ui.SizeType
	{
		return null;
	}
	
	public function getPrefHeight() : ui.SizeType
	{
		return null;
	}
	
}
