package ui;

typedef ResolveSize = { size : Null<Float>, fill : Null<Float> };

/**
 * Handles sizeing of a list of elements.
 */
class Sizer
{
	var spacing : Float;
	var collapseMargin : Bool;
	var vertical : Bool;
	
	public function new( spacing : Float, collapseMargin : Bool, vertical : Bool )
	{
		this.spacing = spacing;
		this.collapseMargin = collapseMargin;
		this.vertical = vertical;
	}
	
	static function nullAdd( a : Null<Float>, b : Null<Float> ) : Null<Float>
	{
		if( a == null )
			return b;
		if( b == null )
			return a;
		return a + b;
	}
	
	static public function max( a : SizeType, b : SizeType )
	{
	}
	
	public function fixSize( st: SizeType, fullP : Float, fullS : Float, totalWeight : Float, available : Float ) : Float
	{
		var rs = resolveSize( fullP, fullS, st );
		var isize = 0.0;
		if( rs.size != null )
			isize += rs.size;
		if( rs.fill != null )
		{
			if( totalWeight == 0 )//special case when one item has weight 0 (a collapsable filler)
				isize = available;
			else
				isize += (rs.fill / totalWeight) * available;
		}
		
		return isize;
	}
	 
	function resolveSize( fullP : Float, fullS : Float, st : SizeType ) : ResolveSize
	{
		var fill : Null<Float> = null;
		var size : Null<Float> = null;
		
		switch( st )
		{
			case Hidden:
				//nothing
			case Fixed( fsize ):
				size = fsize;
			case Ratio( xy ):
				if( vertical )
					size = fullS / xy;
				else
					size = fullS * xy;
			case Fill( weight ):
				fill = weight;
			case FixedFunc( func ):
				size = func();
			case FixedFunc1( func, param ):
				size = func( param );
			case Lines( num ):
				size = FontManager.getDefault().byLines( num );
			case XWidth( num ):
				size = FontManager.getDefault().byXWidth( num );
				
			case Sum( a, b ):
				var rsa = resolveSize( fullP, fullS, a );
				var rsb = resolveSize( fullP, fullS, b );
				
				fill = nullAdd( rsa.fill, rsb.fill );
				size = nullAdd( rsa.size, rsb.size );
				
			case Max( a, b ):
				var rsa = resolveSize( fullP, fullS, a );
				var rsb = resolveSize( fullP, fullS, b );
				
				//NOTE: To determine maximum, only fixed sizes are considered
				if( rsa.size == null )
					return rsb;
				else if( rsb.size == null )
					return rsa;
				else if( rsa.size > rsb.size )
					return rsa;
				else
					return rsb;
		}
		
		return { size : size, fill : fill };
	}

	/**
	 * Recommends a size for the given items, this only considers the primary
	 * dimension
	 */
	public function recommend( items : Array<Dynamic> ) : SizeType
	{
		var size = 0.0;
		var weight = null;
		
		for( _i in items )
		{
			var i = cast( _i, Sizeable );
			var rs = resolveSize( 0, 0, i.getSizeType() );
			size = nullAdd( size, rs.size );
			weight = nullAdd( weight, rs.fill );
		}
		
		//drop space from available (somewhat inverted from code in size)
		size += items.length * spacing;
		if( !collapseMargin )
			size += spacing;	//drop top spacing
		else
			size -= spacing;	//restore final border
		
		//TODO: fill weights are simple ignored now!
		return SizeType.Fixed( size ); 
	}
	
	/**
	 * @param	fullP	[in] the size of the primary dimension
	 * @param	fullS [in] the size of the secondary dimension
	 * @param	items	[in] Iterable<extends SizeType>
	 */
	public function size( fullP : Float, fullS : Float,  items : Array<Dynamic> )
	{
		var available : Float = fullP;
		
		//drop space from available
		available -= items.length * spacing;
		if( !collapseMargin )
			available -= spacing;	//drop top spacing
		else
			available += spacing;	//restore final border
			
		var fillWeight : Float = 0;
		
		for( _i in items )
		{
			var i = cast( _i, Sizeable );
			var rs = resolveSize( fullP, fullS, i.getSizeType() );
			if( rs.size != null )
				available -= rs.size;
			if( rs.fill != null )
				fillWeight += rs.fill;
		}
		
		//size items
		var atP = if( collapseMargin ) 0 else spacing ;
		for( _i in items )
		{
			var i = cast( _i, Sizeable );
			var isize = fixSize( i.getSizeType(), fullP, fullS, fillWeight, available );
			i.setPlacement( atP, isize );
			atP += isize + spacing;
		}
	}
}
