package ui;

import flash.display.Sprite;

/**
 * A base class for Sprite widgets. In cases where possible, use this rather than
 * including the MixinWidget (it will reduce final SWF size).
 */
class SpriteWidget extends Sprite, implements Widget
{
	public function new()
	{
		super();
		MixinWidget();
	}
	
	function _resize( w : Float, h : Float )
	{
		Assert.pureVirtual();
	}
	
	/**
 * Provides the functionality to create a Widget of a Flash Sprite.
 */	
 
	var sizeW : Float;	//our desired size (may not be the same as width)
	var sizeH : Float;
	var enabled : Bool;
		
	/**
	 * Serves as a Ctor and must be called by the including class during its
	 * own construction.
	 */
	function MixinWidget()
	{
		sizeW = 0;
		sizeH = 0;
		enabled = true;
	}
	
	public function move( x : Float, y : Float ) : Void
	{
		this.x = x;
		this.y = y;
	}
	
	//Note that users of this mixin override the _resize function, not "resize" directly...
	public function resize( w : Float, h : Float ) : Void
	{
		sizeW = w;
		sizeH = h;
		_resize( w, h );
	}
	
	public function update( ) : Void
	{
		_resize( sizeW, sizeH );
	}
	
	public function enable( set : Bool ) : Void
	{
		enabled = set;
		update();
	}

	public function getNative() : flash.display.DisplayObjectContainer
	{
		return this;
	}
	

	public function getPrefWidth() : ui.SizeType
	{
		return null;
	}
	
	public function getPrefHeight() : ui.SizeType
	{
		return null;
	}
	
}
