package ui;

import flash.display.Sprite;
import flash.events.Event;

import flash.text.TextField;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;

/**
 * This is to try and provide a better interface to static text fields in Flash.
 * Flash doesn't allow creation of StaticText fields, the ones with which transformations
 * can be done, and the TextField class sucks.
 *
 * TONOTDO: Find a way to load in Character glyphs and do the proper rendering.
 *   ( ha ha ha ha ha ha ha ha ha ha ha ha ha, just not available in Flash9
 *
 * For now this is a simple wrapper to the stupid TextField class.
 *
 * Supported:
 * -different fonts
 * -different colors
 * -clipping to target box
 * -alignment
 * -autosize to height of box (TODO: Add guaranteed fit mode)
 *
 * Not supported:
 *	-rotation (text disappears in flash)
 * -proper scaling (only font-size is supported, no arbitrary scaling)
 * -multiline (only one line for now, our limitation)
 * -cannot removed gutter( fixed at 2 in flash, stupid)
 * -maximum size (flash seems to max out at .size == 127)
 * -minimum size (judging from appearance)
 *
 * NOTE: The text field will fill up the height of the given space based on the font
 * height.  For mot texts this may appear to be misaligned ascenders and descenders
 * are not likely being used, if they were, everything would look more correct.
 */
class StaticText extends Sprite, implements Widget
{
	var tf : TextField;
	public var align(default, setTextAlign) : StaticTextAlign;
	var clipMask : Sprite;
	var suppressLayout : Bool;
	
	public var fillColor : Null<Int>;
	
	public function new( initText : String, ?align : StaticTextAlign, ?clip : Null<Bool> )
	{
		super();
		MixinWidget();
		
		suppressLayout =true;
		
		fillColor = null;	//by default not background
			
		//setup Flash native field
		tf = new TextField();
		tf.selectable = false;
		tf.text = initText;
		addChild( tf );
		
		font = FontManager.getDefault();
		tf.y = -font.getGutter();	//to remove wacky gutter
		
		//setup alignment
		if( align != null )
			this.align = align;
		else
			this.align = StaticTextAlign.Center;
		
		/**
		 *NOTE: Somethign about how this was done before caused Flash to always crash.
		 * I was creating a new object each time, but perhaps it was because I was removing
		 * the clipMaks without clearing it from the textField?
		 * I reduced the situation by not recreated clipping masks, but it is still there, using
		 * them causes the player to crash (during GC'ing it seems, based on timing with
		 * text in Fortress)
		 */
		//setup clipping (is this even needed?, text-test seems to work without)
		if( clip == null )
			clip = false;
		if( clip )
		{
			clipMask = new Sprite();
			addChild( clipMask );
			tf.mask = clipMask;
		}
				
		addEventListener( Event.ADDED, onAdded, false, 0, true /*weak*/ );
		
		suppressLayout = false;
	}
	
	function onAdded( evt : Event )
	{
		if( evt.target != this )
			return;
			
		update();
	}
	
	public function setTextAlign( a : StaticTextAlign ) : StaticTextAlign
	{
		align = a;
		update( );
		return align;
	}
	
	public var text( getText, setText ) : String;	//FAKE
	
	public function setText( t : String ) : String
	{
		//do nothing if its the same (save some cycles)
		if( tf.text == t )
			return t;
			
		tf.text = t;
		update();
		return t;
	}
	
	public function getText( ) : String
	{
		return tf.text;
	}
	
	public var bold( getBold, setBold ) : Bool;	//FAKE
	
	public function setBold( b : Bool) : Bool
	{
		var fmt = new TextFormat();
		fmt.bold = b;
		tf.setTextFormat( fmt );
		update();
		return b;
	}
	
	public function getBold( ) : Bool
	{
		return tf.getTextFormat().bold;
	}
	
	public var textColor( getTextColor, setTextColor ) : Int;	//FAKE
	
	public function setTextColor( c : Int ) : Int
	{
		tf.textColor = c;
		return c;
	}
	
	public function getTextColor( ) : Int
	{
		return tf.textColor;
	}

	public var font( getFont, setFont ) : FontManager;
	
	public function getFont() : FontManager
	{
		return font;
	}
	
	public function setFont( font : FontManager ) : FontManager
	{
		this.font = font;
		
		var fmt = new TextFormat();
		fmt.font = font.getFontName();
		tf.setTextFormat( fmt );
		update();
		
		return font;
	}
	
	function draw( w :  Float, h : Float )
	{
		graphics.clear();
		//graphics.lineStyle( 0, 0 );
		if( fillColor != null )
			graphics.beginFill( fillColor );
		graphics.drawRect( 0, 0, w, h );
	}
	
	/**
	 * The core of the class which determines how the TextField should
	 * actually be formatted.
	 */
	function _resize( w : Float, h : Float )
	{
		if( tf==null || suppressLayout )
			return;
			
		tf.width = w;
		tf.height = h + 2 *font.getGutter();	//accomodate gutters in height (top/bottom)
		draw( w, h );
		
		var fmt = new TextFormat();
		fmt.size = font.fontSizeForHeight( h );
		
		/* START: Flash nonsense workaround */
		switch( align )
		{
			case Left:
				fmt.align = TextFormatAlign.LEFT;
			case Right:
				fmt.align = TextFormatAlign.RIGHT;
			case Center:
				fmt.align = TextFormatAlign.CENTER;
		}
		/* END */
		tf.setTextFormat( fmt );
		
/*		NOTE: This can't be used since Flash is junk when it comes to handling
		dynamic text and scaling.  There is no way to get the *REAL* width of
		the text at this point, based on the new fmt.  We are forced to rely on
		the broken TextAlign mechanism, done above...
		(width will be taken from initial rescale function)
		//Crap, this is of the old font, before setTextFormat is called!!! (or something odd, the width is just wrong!)
		//Though RIGHT Align doesn't work in Flash if width is greater than box... :(
		var tw = tf.getLineMetrics(0).width + 2 * gutter;
		tf.width = tw;
		//trace( tf.text + ":" +  tw );
				
		switch( align )
		{
			case Left:
				tf.x = 0;
			case Right:
				tf.x = w - tw;
			case Center:
				tf.x = (w/2) - (tw/2);
		}
*/

		//clip text to our borders
		if( clipMask != null )
		{
			clipMask.graphics.clear();
			clipMask.graphics.beginFill( 0xFFFFFF );
			clipMask.graphics.drawRect( 0, 0, w, h );
			clipMask.graphics.endFill();
		}
	}
	
	public function getPrefWidth() : SizeType
	{
		//TODO: actual size, not just a guess
		return SizeType.XWidth( tf.text.length );
	}
	
	public function getPrefHeight() : SizeType
	{
		//assuming single-line
		return SizeType.Lines( 1 );
	}

	
	/**
 * Provides the functionality to create a Widget of a Flash Sprite.
 */	
 
	var sizeW : Float;	//our desired size (may not be the same as width)
	var sizeH : Float;
	var enabled : Bool;
		
	/**
	 * Serves as a Ctor and must be called by the including class during its
	 * own construction.
	 */
	function MixinWidget()
	{
		sizeW = 0;
		sizeH = 0;
		enabled = true;
	}
	
	public function move( x : Float, y : Float ) : Void
	{
		this.x = x;
		this.y = y;
	}
	
	//Note that users of this mixin override the _resize function, not "resize" directly...
	public function resize( w : Float, h : Float ) : Void
	{
		sizeW = w;
		sizeH = h;
		_resize( w, h );
	}
	
	public function update( ) : Void
	{
		_resize( sizeW, sizeH );
	}
	
	public function enable( set : Bool ) : Void
	{
		enabled = set;
		update();
	}

	public function getNative() : flash.display.DisplayObjectContainer
	{
		return this;
	}
	
	
}
